#!/usr/bin/perl

# Copyright 2007-2009 Gérald Sédrati-Dinet, gibus@april.org
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, see <http://www.gnu.org/licenses/>.

use strict;
use warnings;
use utf8;
use open ':utf8';

use Sleepycat::DbXml 'simple';
use XML::Twig;
use Getopt::Long;

# Some files will be save under tree hierarchy based on executable location
use FindBin qw($Bin);
use lib "$Bin";
(my $root_dir = $Bin) =~ s/\/[^\/]*$//o;

# Default values
my $path2DbEnv = "$root_dir/dbxml";
my $theContainer = 'en_meps.dbxml';
my $output_dir = "$root_dir/mediawiki/meps_scores";

# Parse command line options
Getopt::Long::Configure("bundling");
GetOptions(
  'path2dbenv|p=s' => \$path2DbEnv,
  'container|c=s'  => \$theContainer,
  'output|o=s'     => \$output_dir,
  'help|h'         =>
    sub {
      print STDERR <<USAGE;
Usage: $0 [-h] [-p <path_to_db_env>] [-c <container>] [-o <output_dir]
Options can be:
  --path2dbenv, -p <path_to_db_env>: path to directory of DB XML database
                                     default: $root_dir/dbxml
  --container, -c <container>:       DB XML container
                                     <container> is typically en_meps (for convenience the final "s" can be omitted, also ".dbxml" can be appended)
                                     default: en_meps.dbxml
  --output, -o <output_dir>:         output directory of mediawiki pages
                                     default: $root_dir/mediawiki/meps_scores
  --help, -h:                        print this message
USAGE
      exit 0;
    }
);
$theContainer =~ s/s?(?:\.dbxml)?$/s/;

my @meps;

my %weights = (
  Software_patents_directive_1st_reading          => {
      WEIGHT   => 1,
      FULLNAME => 'Directive on patentability of "computer-implemented inventions" (software patents), 1st reading',
      DATE     => '20030924',
      ABBREV   => 'Swpats',
                                                    },
  EPLA_resolution                                 => {
      WEIGHT   => 1,
      FULLNAME => 'Resolutions on European Patent Litigation Agreement (EPLA)',
      DATE     => '20061012',
      ABBREV   => 'EPLA  ',
                                                    },
  IPRED2_1st_reading                              => {
      WEIGHT   => 2,
      FULLNAME => 'Criminal measures aimed at ensuring the enforcement of intellectual property rights (IPRED 2), 1st reading',
      DATE     => '20070425',
      ABBREV   => 'IPRED2',
                                                    },
  Cultural_industries_resolution                  => {
      WEIGHT   => 4,
      FULLNAME => 'Rapport Bono on cultural industries in Europe',
      DATE     => '20080410',
      ABBREV   => 'Bono  ',
                                                    },
  Telecoms_package_directives_1st_reading         => {
      WEIGHT   => 8,
      FULLNAME => 'Directives reforming the EU\'s regulatory framework for electronic communications networks and services (telecoms package), 1st reading',
      DATE     => '20080924',
      ABBREV   => 'TP1   ',
                                                    },
  Fundamental_freedoms_on_the_Internet_resolution => {
      WEIGHT   => 4,
      FULLNAME => 'Rapport Lambrinidis on strengthening security and fundamental freedoms on the Internet',
      DATE     => '20090326',
      ABBREV   => 'Lamb. ',
                                                    },
  Telecoms_package_directives_2nd_reading         => {
      WEIGHT   => 8,
      FULLNAME => 'Directives reforming the EU\'s regulatory framework for electronic communications networks and services (telecoms package), 2nd reading',
      DATE     => '20090506',
      ABBREV   => 'TP2   ',
                                                    },
);

my @colours = (
  '#ff0000',
  '#ff1900',
  '#ff3300',
  '#ff4c00',
  '#ff6600',
  '#ff7f00',
  '#ff9900',
  '#ffb200',
  '#ffcc00',
  '#ffe500',
  '#ffff00',
  '#e5ff00',
  '#ccff00',
  '#b2ff00',
  '#99ff00',
  '#7fff00',
  '#66ff00',
  '#4cff00',
  '#33ff00',
  '#19ff00',
  '#00ff00',
);

eval {
    # Open a container in the db environment
    my $env = new DbEnv(0);
    $env->set_cachesize(0, 64 * 1024, 1);
    $env->open($path2DbEnv,
                Db::DB_INIT_MPOOL|Db::DB_CREATE|Db::DB_INIT_LOCK|Db::DB_INIT_LOG);
    my $theMgr = new XmlManager($env);

    my $container = $theMgr->openContainer("$theContainer.dbxml");

    my $query = <<"QUERY";
for \$deputy in collection("dbxml:/$theContainer.dbxml")/politician[container/\@current/string()="true"]
return
  <deputy>
    <name>{ concat("[[", \$deputy/infos/name/wiki/string(), "|", \$deputy/infos/name/full/string(), "]]") }</name>
    <firstname>{ \$deputy/infos/name/first/string() }</firstname>
    <lastname>{ \$deputy/infos/name/last/string() }</lastname>
    <country_code>{ \$deputy/infos/constituency/country/code/string() }</country_code>
    <country_name>{ \$deputy/infos/constituency/country/name/string() }</country_name>
    <grp_code>{ \$deputy/infos/group/abbreviation/string() }</grp_code>
    <grp_name>{ \$deputy/infos/group/name/string() }</grp_name>
    <tel>{ string-join(for \$tel in \$deputy/contact//phone return concat("Tel: [callto://", replace(\$tel/string(), " ", ""), " ", \$tel/string(), "]"), "<br />") }</tel>
        <fax>{ string-join(for \$fax in \$deputy/contact//fax return concat("Fax: ", \$fax/string()), "<br />") }</fax>
    <email>{ string-join(for \$email in \$deputy/contact/email return concat("[mailto:", \$email/string(), " ", replace(\$email/string(), "@", "(à)"), "]"), "<br />") }</email>
    <bxl>{ \$deputy/contact/address[label="Parlement européen"][city="Bruxelles"] }</bxl>
    <str>{ \$deputy/contact/address[label="Parlement européen"][city="Strasbourg Cedex"] }</str>
    {\$deputy/functions}
    {\$deputy/scores}
  </deputy>
QUERY
    #  Perform the query
    eval {
        my $results = $theMgr->query($query);
        my $value;
        while( $results->next($value) ) {
          my $xml = XML::Twig->new(output_encoding => 'UTF-8')->parse($value);
          my $mep = $xml->root;
          my %mep_data;
          $mep_data{NAME}->{WIKI} = $mep->first_child_text('name');
          $mep_data{NAME}->{FIRST} = $mep->first_child_text('firstname');
          $mep_data{NAME}->{LAST} = $mep->first_child_text('lastname');
          $mep_data{COUNTRY}->{CODE} = $mep->first_child_text('country_code');
          $mep_data{COUNTRY}->{NAME} = $mep->first_child_text('country_name');
          $mep_data{GRP}->{CODE} = $mep->first_child_text('grp_code');
          $mep_data{GRP}->{NAME} = $mep->first_child_text('grp_name');
          $mep_data{TEL} = $mep->first_child_text('tel');
          $mep_data{TEL} =~ s/Tel: \[callto:\/\/\+32/Tel Bxl.: [callto:\/\/+32/go;
          $mep_data{TEL} =~ s/Tel: \[callto:\/\/\+33/Tel Str.: [callto:\/\/+33/go;
          $mep_data{FAX} = $mep->first_child_text('fax');
          $mep_data{FAX} =~ s/Fax: \+32/Fax Bxl.: +32/go;
          $mep_data{FAX} =~ s/Fax: \+33/Fax Str.: +33/go;
          $mep_data{EMAIL} = $mep->first_child_text('email');
          my $bxl_addr = $mep->first_child('bxl')->first_child('address');
          $mep_data{BXL} = $bxl_addr->first_child('building')->first_child_text('abbreviation') . ' ' . $bxl_addr->first_child_text('office');
          if ($mep_data{BXL} eq '- Moving') {
            $mep_data{BXL} = '[[MEPs_BXL_SDF|-]]';
          }
          else {
            $mep_data{BXL} =~ s/^(([A-Z][A-Z][A-Z]) (\d\d).+)$/[[MEPs_$2_$3|$1]]/o;
          }

          my $str_addr = $mep->first_child('str')->first_child('address');
          $mep_data{STR} = $str_addr->first_child('building')->first_child_text('abbreviation') . ' ' . $str_addr->first_child_text('office');
          if ($mep_data{STR} eq '- Moving') {
            $mep_data{STR} = '[[MEPs_STR_SDF|-]]';
          }
          else {
            $mep_data{STR} =~ s/^(([A-Z][A-Z][A-Z]) [MT](\d\d).+)$/[[MEPs_$2_$3|$1]]/o;
          }

          my %coms = map {$_->first_child_text('abbreviation') => $_->first_child_text('label')} grep {$_->first_child('abbreviation')} $mep->first_child('functions')->children('function');
          $mep_data{COMS} = \%coms;

          my %scores;
          if (defined $mep->first_child('scores')) {
            foreach my $score ($mep->first_child('scores')->children('score')){
              my $vote = $score->first_child_text('wiki');
              my $value = $score->first_child_text('value');

              $scores{$vote} = $value;
            }
          }

          my $total_score = 0;
          my $total_weight = 0;
          $mep_data{SCORE} = '';
          foreach my $vote (sort {$weights{$a}->{DATE} <=> $weights{$b}->{DATE}} keys %weights) {
            my $value = exists $scores{$vote} ? $scores{$vote} : 50;
            $scores{$vote} = '-' unless exists $scores{$vote};
            $mep_data{SCORE} .= '<br /><small>' . $weights{$vote}->{ABBREV} . ': ' . $scores{$vote} . '</small>';
            $total_score += ($value * $weights{$vote}->{WEIGHT});
            $total_weight += $weights{$vote}->{WEIGHT};
          }

          my @no_votes = grep {$_ eq '-'} values %scores;
          $mep_data{TOTAL_SCORE} = (scalar(@no_votes) == scalar(keys %scores)) ?  -1 :
            sprintf("%.1f", $total_score / $total_weight);
          $mep_data{SCORE} = '<strong>' . ($mep_data{TOTAL_SCORE} == -1 ? 'No recorded vote' : $mep_data{TOTAL_SCORE}) . '</strong>' . $mep_data{SCORE};

          push @meps, \%mep_data;
        }
	  };

    if (my $e = catch std::exception) {
      warn "Query $query failed\n";
      warn $e->what() . "\n";
      exit( -1 );
    }
    elsif ($@) {
      warn "Query $query failed\n";
      warn $@;
      exit( -1 );
    }
};

if (my $e = catch std::exception) {
  warn "Query failed\n";
  warn $e->what() . "\n";
  exit( -1 );
}
elsif ($@) {
  warn "Query failed\n";
  warn $@;
  exit( -1 );
}

# Build mediawiki page by score
my $mediawiki_page = << "MEDIAWIKI_PAGE";
__NOEDITSECTION__
<div class="LanguageLinks"><table width="100%"><tr valign="top" style="background: #EEF3E2"><td style="width: 25px; padding-left: 0.5em;">[[Image:Geographylogo.png|25px|Languages]]</td><td style="padding: 1px 1em 0; background: #F6F9ED;">'''[[MEPs_by_score|English]]''' - '''[[Eurodeputes_par_note|Français]]'''</td></tr></table></div>

==[[Political_Memory|Political Memory]]: MEPs ranking by score==

The ranking below has been computed with MEPs scores on the following votes and with the following coefficients:

MEDIAWIKI_PAGE

foreach my $vote (sort {$weights{$a}->{DATE} <=> $weights{$b}->{DATE}} keys %weights) {
  $mediawiki_page .= "* $weights{$vote}->{ABBREV}: [[$vote|$weights{$vote}->{FULLNAME}]], coeff. $weights{$vote}->{WEIGHT}\n";
}

$mediawiki_page .= << "MEDIAWIKI_PAGE";

You can also view this ranking:
* [[MEPs_score_by_name|by alphabetical order]] ;
* [[MEPs_score_by_country|by country]] ;
* [[MEPs_score_by_group|by political group]] ;
* [[MEPs_score_by_committee|by committee]].

MEDIAWIKI_PAGE

$mediawiki_page .= "{| border='1' cellpadding='5' cellspacing='0' class='wikitable sortable'\n";
$mediawiki_page .= "! Rank !! MEP !! Scores !! Country !! Group !! Contact\n";
$mediawiki_page .= "|-\n";
my $rank = 1;
foreach my $deputy (sort {$b->{TOTAL_SCORE} <=> $a->{TOTAL_SCORE} || $a->{NAME}->{LAST} cmp $b->{NAME}->{LAST} || $a->{NAME}->{FIRST} cmp $b->{NAME}->{FIRST}} @meps) {
  (my $grp_wiki = $deputy->{GRP}->{CODE}) =~ s/[\/-].+$//o;
  $mediawiki_page .= "| $rank || $deputy->{NAME}->{WIKI} || style=\"background:".($deputy->{TOTAL_SCORE} == -1 ? '#ffffff' : $colours[sprintf("%.0f", $deputy->{TOTAL_SCORE}/5)])."\" | $deputy->{SCORE} || {{#icon:$deputy->{COUNTRY}->{CODE}.png|$deputy->{COUNTRY}->{NAME}||MEPs_score_$deputy->{COUNTRY}->{CODE}}} [[MEPs_score_$deputy->{COUNTRY}->{CODE}|$deputy->{COUNTRY}->{CODE}]] || {{#icon:$grp_wiki.png|$deputy->{GRP}->{CODE}||MEPs_score_$grp_wiki}} [[MEPs_score_$grp_wiki|$deputy->{GRP}->{CODE}]] || $deputy->{TEL}<br />$deputy->{FAX}<br />$deputy->{EMAIL}\n";
  $mediawiki_page .= "|-\n";
  $deputy->{RANK} = $rank;
  $rank++;
}
$mediawiki_page =~ s/\|-\n$/|}\n/so;
$mediawiki_page .= "\n----\n\nBack to home page of [[Political_Memory|Political Memory]]\n";
$mediawiki_page .= "[[Category:MEPs]] [[Category:Political_Memory]]\n";

# Output mediawiki page by score
my $mediawiki_location = "$output_dir/MEPs_by_score.mediawiki";
open MEDIAWIKI, ">$mediawiki_location" or die "Cannot write in file $mediawiki_location: $!\n";
print MEDIAWIKI $mediawiki_page;
close MEDIAWIKI or die "Error when closing $mediawiki_location: $!\n";
warn "$mediawiki_location done\n";

# Build mediawiki page by name
$mediawiki_page = << "MEDIAWIKI_PAGE";
__NOEDITSECTION__
<div class="LanguageLinks"><table width="100%"><tr valign="top" style="background: #EEF3E2"><td style="width: 25px; padding-left: 0.5em;">[[Image:Geographylogo.png|25px|Languages]]</td><td style="padding: 1px 1em 0; background: #F6F9ED;">'''[[MEPs_score_by_name|English]]''' - '''[[Eurodeputes_note_par_nom|Français]]'''</td></tr></table></div>

==[[Political_Memory|Political Memory]]: MEPs ranking by name==

The ranking below has been computed with MEPs scores on the following votes and with the following coefficients:

MEDIAWIKI_PAGE

foreach my $vote (sort {$weights{$a}->{DATE} <=> $weights{$b}->{DATE}} keys %weights) {
  $mediawiki_page .= "* $weights{$vote}->{ABBREV}: [[$vote|$weights{$vote}->{FULLNAME}]], coeff. $weights{$vote}->{WEIGHT}\n";
}

$mediawiki_page .= << "MEDIAWIKI_PAGE";

You can also view this ranking:
* [[MEPs_by_score|by score]] ;
* [[MEPs_score_by_country|by country]] ;
* [[MEPs_score_by_group|by political group]] ;
* [[MEPs_score_by_committee|by committee]].

MEDIAWIKI_PAGE

$mediawiki_page .= "{| border='1' cellpadding='5' cellspacing='0' class='wikitable sortable'\n";
$mediawiki_page .= "! Rank !! MEP !! Scores !! Country !! Group !! Contact\n";
$mediawiki_page .= "|-\n";
foreach my $deputy ( sort {$a->{NAME}->{LAST} cmp $b->{NAME}->{LAST} || $a->{NAME}->{FIRST} cmp $b->{NAME}->{FIRST}} @meps) {
  (my $grp_wiki = $deputy->{GRP}->{CODE}) =~ s/[\/-].+$//o;
  $mediawiki_page .= "| $deputy->{RANK} || $deputy->{NAME}->{WIKI} || style=\"background:".($deputy->{TOTAL_SCORE} == -1 ? '#ffffff' : $colours[sprintf("%.0f", $deputy->{TOTAL_SCORE}/5)])."\" | $deputy->{SCORE} || {{#icon:$deputy->{COUNTRY}->{CODE}.png|$deputy->{COUNTRY}->{NAME}||MEPs_score_$deputy->{COUNTRY}->{CODE}}} [[MEPs_score_$deputy->{COUNTRY}->{CODE}|$deputy->{COUNTRY}->{CODE}]] || {{#icon:$grp_wiki.png|$deputy->{GRP}->{CODE}||MEPs_score_$grp_wiki}} [[MEPs_score_$grp_wiki|$deputy->{GRP}->{CODE}]] || $deputy->{TEL}<br />$deputy->{FAX}<br />$deputy->{EMAIL}\n";
  $mediawiki_page .= "|-\n";
}
$mediawiki_page =~ s/\|-\n$/|}\n/so;
$mediawiki_page .= "\n----\n\nBack to home page of [[Political_Memory|Political Memory]]\n";
$mediawiki_page .= "[[Category:MEPs]] [[Category:Political_Memory]]\n";

# Output mediawiki page by name
$mediawiki_location = "$output_dir/MEPs_score_by_name.mediawiki";
open MEDIAWIKI, ">$mediawiki_location" or die "Cannot write in file $mediawiki_location: $!\n";
print MEDIAWIKI $mediawiki_page;
close MEDIAWIKI or die "Error when closing $mediawiki_location: $!\n";
warn "$mediawiki_location done\n";

# Build mediawiki pages for each country
my %country_meps = map {$_->{COUNTRY}->{CODE} => $_->{COUNTRY}->{NAME}} @meps;
my %country_mean;
foreach my $code (sort keys %country_meps) {
  my $mediawiki_page = << "MEDIAWIKI_PAGE";
__NOEDITSECTION__
<div class="LanguageLinks"><table width="100%"><tr valign="top" style="background: #EEF3E2"><td style="width: 25px; padding-left: 0.5em;">[[Image:Geographylogo.png|25px|Languages]]</td><td style="padding: 1px 1em 0; background: #F6F9ED;">'''[[MEPs_score_$code|English]]''' - '''[[Eurodeputes_note_$code|Français]]'''</td></tr></table></div>

==[[Political_Memory|Political Memory]]: $country_meps{$code} {{#icon:$code.png|$country_meps{$code}}}, MEPs ranking==

The ranking below has been computed with MEPs scores on the following votes and with the following coefficients:

MEDIAWIKI_PAGE

  foreach my $vote (sort {$weights{$a}->{DATE} <=> $weights{$b}->{DATE}} keys %weights) {
    $mediawiki_page .= "* $weights{$vote}->{ABBREV}: [[$vote|$weights{$vote}->{FULLNAME}]], coeff. $weights{$vote}->{WEIGHT}\n";
  }

  my $country_nb = 0;
  my $country_total = 0;
  $mediawiki_page .= "\n{| border='1' cellpadding='5' cellspacing='0' class='wikitable sortable'\n";
  $mediawiki_page .= "! Rank !! MEP !! Scores !! Group !! Contact\n";
  $mediawiki_page .= "|-\n";
  foreach my $deputy ( sort {$b->{TOTAL_SCORE} <=> $a->{TOTAL_SCORE}} grep {$_->{COUNTRY}->{CODE} eq $code} @meps) {
    (my $grp_wiki = $deputy->{GRP}->{CODE}) =~ s/[\/-].+$//o;
    $mediawiki_page .= "| $deputy->{RANK} || $deputy->{NAME}->{WIKI} || style=\"background:".($deputy->{TOTAL_SCORE} == -1 ? '#ffffff' : $colours[sprintf("%.0f", $deputy->{TOTAL_SCORE}/5)])."\" | $deputy->{SCORE} || {{#icon:$grp_wiki.png|$deputy->{GRP}->{CODE}||MEPs_score_$grp_wiki}} [[MEPs_score_$grp_wiki|$deputy->{GRP}->{CODE}]] || $deputy->{TEL}<br />$deputy->{FAX}<br />$deputy->{EMAIL}\n";
    $mediawiki_page .= "|-\n";
    $country_nb++ if $deputy->{TOTAL_SCORE} > -1;
    $country_total += $deputy->{TOTAL_SCORE} if $deputy->{TOTAL_SCORE} > -1;
  }
  $mediawiki_page =~ s/\|-\n$/|}\n/so;
  $mediawiki_page .= "\n----\n\n[[MEPs_score_by_country|Ranking for other countries]]\n";
  $mediawiki_page .= "[[Category:MEPs]] [[Category:Political_Memory]]\n";
  $country_mean{$code} = sprintf("%.1f", $country_total / $country_nb);

  # Output mediawiki page for this country
  my $mediawiki_location = "$output_dir/MEPs_score_$code.mediawiki";
  open MEDIAWIKI, ">$mediawiki_location" or die "Cannot write in file $mediawiki_location: $!\n";
  print MEDIAWIKI $mediawiki_page;
  close MEDIAWIKI or die "Error when closing $mediawiki_location: $!\n";
  warn "$mediawiki_location done\n";
}

# Build mediawiki page by country
$mediawiki_page = << "MEDIAWIKI_PAGE";
__NOEDITSECTION__
<div class="LanguageLinks"><table width="100%"><tr valign="top" style="background: #EEF3E2"><td style="width: 25px; padding-left: 0.5em;">[[Image:Geographylogo.png|25px|Languages]]</td><td style="padding: 1px 1em 0; background: #F6F9ED;">'''[[MEPs_score_by_country|English]]''' - '''[[Eurodeputes_note_par_pays|Français]]'''</td></tr></table></div>

==[[Political_Memory|Political Memory]]: MEPs ranking by country==

The rankings linked below have been computed with MEPs scores on the following votes and with the following:

MEDIAWIKI_PAGE

foreach my $vote (sort {$weights{$a}->{DATE} <=> $weights{$b}->{DATE}} keys %weights) {
  $mediawiki_page .= "* $weights{$vote}->{ABBREV}: [[$vote|$weights{$vote}->{FULLNAME}]], coeff. $weights{$vote}->{WEIGHT}\n";
}

$mediawiki_page .= << "MEDIAWIKI_PAGE";

You can also view this ranking:
* [[MEPs_by_score|by score]] ;
* [[MEPs_score_by_name|by alphabetical order]] ;
* [[MEPs_score_by_group|by political group]] ;
* [[MEPs_score_by_committee|by committee]].

{|- style="background:none;"
MEDIAWIKI_PAGE

my $nb_col = 1;
my $col = 0;
foreach my $country_code (sort  {$country_mean{$b} <=> $country_mean{$a}} keys %country_meps) {
  $mediawiki_page .= "| {{#icon:$country_code.png|$country_meps{$country_code}||MEPs_score_$country_code}} [[MEPs_score_$country_code|$country_meps{$country_code}]]: mean score: $country_mean{$country_code}/100\n";
  $col++;
  if ($col == $nb_col) {
    $mediawiki_page .= "|-\n";
    $col = 0;
  }
}
$mediawiki_page =~ s/|-\n$//o;
$mediawiki_page .= "|}\n";
$mediawiki_page .= "\n----\n\nBack to home page of [[Political_Memory|Political Memory]]\n";
$mediawiki_page .= "[[Category:MEPs]] [[Category:Political_Memory]]\n";

# Output mediawiki page by country
$mediawiki_location = "$output_dir/MEPs_score_by_country.mediawiki";
open MEDIAWIKI, ">$mediawiki_location" or die "Cannot write in file $mediawiki_location: $!\n";
print MEDIAWIKI $mediawiki_page;
close MEDIAWIKI or die "Error when closing $mediawiki_location: $!\n";
warn "$mediawiki_location done\n";

# Build mediawiki pages for each group
my %group_meps = map {my $grp = $_->{GRP}->{CODE}; $grp =~ s/[\/-].+$//o; $grp => $_->{GRP}->{NAME}} @meps;
my %group_mean;
foreach my $code (sort keys %group_meps) {
  my $mediawiki_page = << "MEDIAWIKI_PAGE";
__NOEDITSECTION__
<div class="LanguageLinks"><table width="100%"><tr valign="top" style="background: #EEF3E2"><td style="width: 25px; padding-left: 0.5em;">[[Image:Geographylogo.png|25px|Languages]]</td><td style="padding: 1px 1em 0; background: #F6F9ED;">'''[[MEPs_score_$code|English]]''' - '''[[Eurodeputes_note_$code|Français]]'''</td></tr></table></div>

==[[Political_Memory|Political Memory]]: $group_meps{$code} {{#icon:$code.png|$group_meps{$code}}}, MEPs ranking==

The ranking below has been computed with MEPs scores on the following votes and with the following coefficients:

MEDIAWIKI_PAGE

  foreach my $vote (sort {$weights{$a}->{DATE} <=> $weights{$b}->{DATE}} keys %weights) {
    $mediawiki_page .= "* $weights{$vote}->{ABBREV}: [[$vote|$weights{$vote}->{FULLNAME}]], coeff. $weights{$vote}->{WEIGHT}\n";
  }

  my $group_nb = 0;
  my $group_total = 0;
  $mediawiki_page .= "\n{| border='1' cellpadding='5' cellspacing='0' class='wikitable sortable'\n";
  $mediawiki_page .= "! Rank !! MEP !! Scores !! Country !! Contact\n";
  $mediawiki_page .= "|-\n";
  foreach my $deputy ( sort {$b->{TOTAL_SCORE} <=> $a->{TOTAL_SCORE}} grep {$_->{GRP}->{CODE} =~ /^$code/} @meps) {
    $mediawiki_page .= "| $deputy->{RANK} || $deputy->{NAME}->{WIKI} || style=\"background:".($deputy->{TOTAL_SCORE} == -1 ? '#ffffff' : $colours[sprintf("%.0f", $deputy->{TOTAL_SCORE}/5)])."\" | $deputy->{SCORE} || {{#icon:$deputy->{COUNTRY}->{CODE}.png|$deputy->{COUNTRY}->{NAME}||MEPs_score_$deputy->{COUNTRY}->{CODE}}} [[MEPs_score_$deputy->{COUNTRY}->{CODE}|$deputy->{COUNTRY}->{CODE}]] || $deputy->{TEL}<br />$deputy->{FAX}<br />$deputy->{EMAIL}\n";
    $mediawiki_page .= "|-\n";
    $group_nb++ if $deputy->{TOTAL_SCORE} > -1;
    $group_total += $deputy->{TOTAL_SCORE} if $deputy->{TOTAL_SCORE} > -1;
  }
  $mediawiki_page =~ s/\|-\n$/|}\n/so;
  $mediawiki_page .= "\n----\n\n[[MEPs_score_by_group|Ranking for other political groups]]\n";
  $mediawiki_page .= "[[Category:MEPs]] [[Category:Political_Memory]]\n";
  $group_mean{$code} = sprintf("%.1f", $group_total / $group_nb);

  # Output mediawiki page for this group
  my $mediawiki_location = "$output_dir/MEPs_score_$code.mediawiki";
  open MEDIAWIKI, ">$mediawiki_location" or die "Cannot write in file $mediawiki_location: $!\n";
  print MEDIAWIKI $mediawiki_page;
  close MEDIAWIKI or die "Error when closing $mediawiki_location: $!\n";
  warn "$mediawiki_location done\n";
}

# Build mediawiki page by group
$mediawiki_page = << "MEDIAWIKI_PAGE";
__NOEDITSECTION__
<div class="LanguageLinks"><table width="100%"><tr valign="top" style="background: #EEF3E2"><td style="width: 25px; padding-left: 0.5em;">[[Image:Geographylogo.png|25px|Languages]]</td><td style="padding: 1px 1em 0; background: #F6F9ED;">'''[[MEPs_score_by_group|English]]''' - '''[[Eurodeputes_note_par_groupe|Français]]'''</td></tr></table></div>

==[[Political_Memory|Political Memory]]: MEPs ranking by political group==

The rankings linked below have been computed with MEPs scores on the following votes and with the following:

MEDIAWIKI_PAGE

foreach my $vote (sort {$weights{$a}->{DATE} <=> $weights{$b}->{DATE}} keys %weights) {
  $mediawiki_page .= "* $weights{$vote}->{ABBREV}: [[$vote|$weights{$vote}->{FULLNAME}]], coeff. $weights{$vote}->{WEIGHT}\n";
}

$mediawiki_page .= << "MEDIAWIKI_PAGE";

You can also view this ranking:
* [[MEPs_by_score|by score]] ;
* [[MEPs_score_by_name|by alphabetical order]] ;
* [[MEPs_score_by_country|by country]] ;
* [[MEPs_score_by_committee|by committee]].

{|- style="background:none;"
MEDIAWIKI_PAGE

$nb_col = 1;
$col = 0;
foreach my $group_code (sort {$group_mean{$b} <=> $group_mean{$a}} keys %group_meps) {
  $mediawiki_page .= "| {{#icon:$group_code.png|$group_meps{$group_code}||MEPs_score_$group_code}}\n";
  $mediawiki_page .= "| [[MEPs_score_$group_code|$group_meps{$group_code}]]: mean score: $group_mean{$group_code}/100\n";
  $col++;
  if ($col == $nb_col) {
    $mediawiki_page .= "|-\n";
    $col = 0;
  }
}
$mediawiki_page =~ s/|-\n$//o;
$mediawiki_page .= "|}\n";
$mediawiki_page .= "\n----\n\nBack to home page of [[Political_Memory|Political Memory]]\n";
$mediawiki_page .= "[[Category:MEPs]] [[Category:Political_Memory]]\n";

# Output mediawiki page by group
$mediawiki_location = "$output_dir/MEPs_score_by_group.mediawiki";
open MEDIAWIKI, ">$mediawiki_location" or die "Cannot write in file $mediawiki_location: $!\n";
print MEDIAWIKI $mediawiki_page;
close MEDIAWIKI or die "Error when closing $mediawiki_location: $!\n";
warn "$mediawiki_location done\n";

# Build mediawiki pages for each committee
my %com_meps = map {%{$_->{COMS}}} grep {exists $_->{COMS}} @meps;
my %com_mean;
foreach my $code (sort keys %com_meps) {
  my $mediawiki_page = << "MEDIAWIKI_PAGE";
__NOEDITSECTION__
<div class="LanguageLinks"><table width="100%"><tr valign="top" style="background: #EEF3E2"><td style="width: 25px; padding-left: 0.5em;">[[Image:Geographylogo.png|25px|Languages]]</td><td style="padding: 1px 1em 0; background: #F6F9ED;">'''[[MEPs_score_$code|English]]''' - '''[[Eurodeputes_note_$code|Français]]'''</td></tr></table></div>

==[[Political_Memory|Political Memory]]: $com_meps{$code}, MEPs ranking==

The ranking below has been computed with MEPs scores on the following votes and with the following coefficients:

MEDIAWIKI_PAGE

  foreach my $vote (sort {$weights{$a}->{DATE} <=> $weights{$b}->{DATE}} keys %weights) {
    $mediawiki_page .= "* $weights{$vote}->{ABBREV}: [[$vote|$weights{$vote}->{FULLNAME}]], coeff. $weights{$vote}->{WEIGHT}\n";
  }

  my $com_nb = 0;
  my $com_total = 0;
  $mediawiki_page .= "\n{| border='1' cellpadding='5' cellspacing='0' class='wikitable sortable'\n";
  $mediawiki_page .= "! Rank !! MEP !! Scores !! Country !! Group !! Contact\n";
  $mediawiki_page .= "|-\n";
  foreach my $deputy ( sort {$b->{TOTAL_SCORE} <=> $a->{TOTAL_SCORE}} grep {$_->{COMS} and exists $_->{COMS}->{$code}} @meps) {
    (my $grp_wiki = $deputy->{GRP}->{CODE}) =~ s/[\/-].+$//o;
    $mediawiki_page .= "| $deputy->{RANK} || $deputy->{NAME}->{WIKI} || style=\"background:".($deputy->{TOTAL_SCORE} == -1 ? '#ffffff': $colours[sprintf("%.0f", $deputy->{TOTAL_SCORE}/5)])."\" | $deputy->{SCORE} || {{#icon:$deputy->{COUNTRY}->{CODE}.png|$deputy->{COUNTRY}->{NAME}||MEPs_score_$deputy->{COUNTRY}->{CODE}}} [[MEPs_score_$deputy->{COUNTRY}->{CODE}|$deputy->{COUNTRY}->{CODE}]] || {{#icon:$grp_wiki.png|$deputy->{GRP}->{CODE}||MEPs_score_$grp_wiki}} [[MEPs_score_$grp_wiki|$deputy->{GRP}->{CODE}]] || $deputy->{TEL}<br />$deputy->{FAX}<br />$deputy->{EMAIL}\n";
    $mediawiki_page .= "|-\n";
    $com_nb++ if $deputy->{TOTAL_SCORE} > -1;
    $com_total += $deputy->{TOTAL_SCORE} if $deputy->{TOTAL_SCORE} > -1;
  }
  $mediawiki_page =~ s/\|-\n$/|}\n/so;
  $mediawiki_page .= "\n----\n\n[[MEPs_score_by_committee|Ranking for other committees]]\n";
  $mediawiki_page .= "[[Category:MEPs]] [[Category:Political_Memory]]\n";
  $com_mean{$code} = sprintf("%.1f", $com_total / $com_nb);

  # Output mediawiki page for this committee
  my $mediawiki_location = "$output_dir/MEPs_score_$code.mediawiki";
  open MEDIAWIKI, ">$mediawiki_location" or die "Cannot write in file $mediawiki_location: $!\n";
  print MEDIAWIKI $mediawiki_page;
  close MEDIAWIKI or die "Error when closing $mediawiki_location: $!\n";
  warn "$mediawiki_location done\n";
}

# Build mediawiki page by committee
$mediawiki_page = << "MEDIAWIKI_PAGE";
__NOEDITSECTION__
<div class="LanguageLinks"><table width="100%"><tr valign="top" style="background: #EEF3E2"><td style="width: 25px; padding-left: 0.5em;">[[Image:Geographylogo.png|25px|Languages]]</td><td style="padding: 1px 1em 0; background: #F6F9ED;">'''[[MEPs_score_by_committee|English]]''' - '''[[Eurodeputes_note_par_commission|Français]]'''</td></tr></table></div>

==[[Political_Memory|Political Memory]]: MEPs ranking by committee==

The rankings linked below have been computed with MEPs scores on the following votes and with the following:

MEDIAWIKI_PAGE

foreach my $vote (sort {$weights{$a}->{DATE} <=> $weights{$b}->{DATE}} keys %weights) {
  $mediawiki_page .= "* $weights{$vote}->{ABBREV}: [[$vote|$weights{$vote}->{FULLNAME}]], coeff. $weights{$vote}->{WEIGHT}\n";
}

$mediawiki_page .= << "MEDIAWIKI_PAGE";

You can also view this ranking:
* [[MEPs_by_score|by score]] ;
* [[MEPs_score_by_name|by alphabetical order]] ;
* [[MEPs_score_by_country|by country]] ;
* [[MEPs_score_by_group|by political group]].

MEDIAWIKI_PAGE

$mediawiki_page .= join "\n", map {"* [[MEPs_score_$_|$com_meps{$_}]]: mean score: $com_mean{$_}/100"} sort {$com_mean{$b} <=> $com_mean{$a}} keys %com_meps;
$mediawiki_page .= "\n----\n\nBack to home page of [[Political_Memory|Political Memory]]\n";
$mediawiki_page .= "[[Category:MEPs]] [[Category:Political_Memory]]\n";

# Output mediawiki page by committee
$mediawiki_location = "$output_dir/MEPs_score_by_committee.mediawiki";
open MEDIAWIKI, ">$mediawiki_location" or die "Cannot write in file $mediawiki_location: $!\n";
print MEDIAWIKI $mediawiki_page;
close MEDIAWIKI or die "Error when closing $mediawiki_location: $!\n";
warn "$mediawiki_location done\n";
