#!/usr/bin/perl

# Copyright 2007-2008 Gérald Sédrati-Dinet, gibus@april.org
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, see <http://www.gnu.org/licenses/>.

use strict;
use warnings;
use utf8;
use open ':utf8';

use Sleepycat::DbXml 'simple';
use XML::Twig;
use Getopt::Long;

# Some files will be save under tree hierarchy based on executable location
use FindBin qw($Bin);
use lib "$Bin";
(my $root_dir = $Bin) =~ s/\/[^\/]*$//o;

# Default values
my $path2DbEnv = "$root_dir/dbxml";
my $theContainer = 'en_meps.dbxml';
my $theVoteContainer = 'en_votes.dbxml';
my $output_dir = "$root_dir/mediawiki/meps_committees";

# Parse command line options
Getopt::Long::Configure("bundling");
GetOptions(
  'path2dbenv|p=s'      => \$path2DbEnv,
  'container|c=s'       => \$theContainer,
  'vote_container|v=s'  => \$theVoteContainer,
  'output|o=s'          => \$output_dir,
  'help|h'              =>
    sub {
      print STDERR <<USAGE;
Usage: $0 [-h] [-p <path_to_db_env>] [-c <container>] [-o <output_dir]
Options can be:
  --path2dbenv, -p <path_to_db_env>: path to directory of DB XML database
                                     default: $root_dir/dbxml
  --container, -c <container>:       DB XML container
                                     <container> is typically en_meps (for convenience the final "s" can be omitted, also ".dbxml" can be appended)
                                     default: en_meps.dbxml
  --vote_container, -c <container>:  DB XML container for votes
                                     <vote_container> is typically en_votes (for convenience the final "s" can be omitted, also ".dbxml" can be appended)
                                     default: en_votes.dbxml
  --output, -o <output_dir>:         output directory of mediawiki pages
                                     default: $root_dir/mediawiki/meps_committees
  --help, -h:                        print this message
USAGE
      exit 0;
    }
);
$theContainer =~ s/s?(?:\.dbxml)?$/s/;
$theVoteContainer =~ s/s?(?:\.dbxml)?$/s/;

my @meps;
my @colours = (
  '#ff0000',
  '#ff1900',
  '#ff3300',
  '#ff4c00',
  '#ff6600',
  '#ff7f00',
  '#ff9900',
  '#ffb200',
  '#ffcc00',
  '#ffe500',
  '#ffff00',
  '#e5ff00',
  '#ccff00',
  '#b2ff00',
  '#99ff00',
  '#7fff00',
  '#66ff00',
  '#4cff00',
  '#33ff00',
  '#19ff00',
  '#00FF00',
);

eval {
    # Open containers in the db environment
    my $env = new DbEnv(0);
    $env->set_cachesize(0, 64 * 1024, 1);
    $env->open($path2DbEnv,
                Db::DB_INIT_MPOOL|Db::DB_CREATE|Db::DB_INIT_LOCK|Db::DB_INIT_LOG);
    my $theMgr = new XmlManager($env);

    my $container = $theMgr->openContainer("$theContainer.dbxml");
    my $vote_container = $theMgr->openContainer("$theVoteContainer.dbxml");

    # Get Telecoms Package vote
    my $vote_document = $vote_container->getDocument('Telecoms_package_directives_1st_reading');
    my $vote_doc_string = $vote_document->getContent();
    my $vote_xml = XML::Twig->new(output_encoding=>'UTF-8',pretty_print => 'indented')->parse($vote_doc_string);
    my %am138;
    foreach my $vote ($vote_xml->root->children('vote')) {
        next unless $vote->first_child('subject')->first_child_text('part') eq 'am. 138/1';
        foreach my $mep ($vote->first_child('result')->children('mep')) {
            $am138{$mep->first_child_text('dbxmlid')} = $mep->first_child_text('choice');
        }
        last;
    }

    # Query for ITRE MEPs
    my $query = <<"QUERY";
  <result>
    {
    for \$deputy in collection("dbxml:/$theContainer.dbxml")/politician[container/\@current/string()="true" and functions/function/abbreviation/string()="ITRE"]
    order by \$deputy/infos/constituency/country/code/string(), \$deputy/infos/group/abbreviation/string(), upper-case(\$deputy/infos/name/last/string()), upper-case(\$deputy/infos/name/first/string())
    return
      <deputy>
        <role>{ \$deputy/functions/function[abbreviation/string()="ITRE"]/role/string() }</role>
        <name>{ concat("[[", \$deputy/infos/name/wiki/string(), "|", \$deputy/infos/name/full/string(), "]]") }</name>
        <mepid>{ \$deputy/infos/name/wiki/string() }</mepid>
        <country>{ concat("{{#icon:", \$deputy/infos/constituency/country/code/string(), ".png|", \$deputy/infos/constituency/country/name/string(), "||MEPs_", \$deputy/infos/constituency/country/code/string(), "}} [[MEPs_", \$deputy/infos/constituency/country/code/string(), "|", \$deputy/infos/constituency/country/name/string(), "]]") }</country>
        <country_name>{ \$deputy/infos/constituency/country/name/string() }</country_name>
        <country_code>{ \$deputy/infos/constituency/country/code/string() }</country_code>
        <grp>{ \$deputy/infos/group/abbreviation/string() }</grp>
        <tel>{ string-join(for \$tel in \$deputy/contact//phone return concat("Tel: [callto://", replace(\$tel/string(), " ", ""), " ", \$tel/string(), "]"), "<br />") }</tel>
        <fax>{ string-join(for \$fax in \$deputy/contact//fax return concat("Fax: ", \$fax/string()), "<br />") }</fax>
        <email>{ string-join(for \$email in \$deputy/contact/email return concat("[mailto:", \$email/string(), " ", replace(\$email/string(), "@", " (at)"), "]"), "<br />") }</email>
        <bxl>{ \$deputy/contact/address[label="Parlement européen"][city="Bruxelles"] }</bxl>
        <str>{ \$deputy/contact/address[label="Parlement européen"][city="Strasbourg Cedex"] }</str>
        <score>{ \$deputy/scores/score[wiki="Telecoms_package_directives_1st_reading"]/value/string() }</score>
      </deputy>
    }
  </result>
QUERY
    #  Perform the query
    eval {
        my $results = $theMgr->query($query);
        my $value;
        while( $results->next($value) ) {
          my $xml = XML::Twig->new(output_encoding => 'UTF-8')->parse($value);
          my $code =  $xml->root->first_child_text('code');
          foreach my $mp ($xml->root->children('deputy')) {
            my %mp_data;
            $mp_data{NAME} = $mp->first_child_text('name');
            $mp_data{COUNTRY} = $mp->first_child_text('country');
            $mp_data{GRP} = $mp->first_child_text('grp');
            $mp_data{TEL} = $mp->first_child_text('tel');
            $mp_data{TEL} =~ s/Tel: \[callto:\/\/\+32/Tel Bxl.: [callto:\/\/+32/go;
            $mp_data{TEL} =~ s/Tel: \[callto:\/\/\+33/Tel Str.: [callto:\/\/+33/go;
            $mp_data{FAX} = $mp->first_child_text('fax');
            $mp_data{FAX} =~ s/Fax: \+32/Fax Bxl.: +32/go;
            $mp_data{FAX} =~ s/Fax: \+33/Fax Str.: +33/go;
            $mp_data{EMAIL} = $mp->first_child_text('email');
            my $bxl_addr = $mp->first_child('bxl')->first_child('address');
            $mp_data{BXL} = $bxl_addr->first_child('building')->first_child_text('abbreviation') . ' ' . $bxl_addr->first_child_text('office');
            if ($mp_data{BXL} eq '- Moving') {
              $mp_data{BXL} = '[[MEPs_BXL_SDF|-]]';
            }
            else {
              $mp_data{BXL} =~ s/^(([A-Z][A-Z][A-Z]) (\d\d).+)$/[[MEPs_$2_$3|$1]]/o;
            }

            my $str_addr = $mp->first_child('str')->first_child('address');
            $mp_data{STR} = $str_addr->first_child('building')->first_child_text('abbreviation') . ' ' . $str_addr->first_child_text('office');
            if ($mp_data{STR} eq '- Moving') {
              $mp_data{STR} = '[[MEPs_STR_SDF|-]]';
            }
            else {
              $mp_data{STR} =~ s/^(([A-Z][A-Z][A-Z]) [MT](\d\d).+)$/[[MEPs_$2_$3|$1]]/o;
            }

            my $role = $mp->first_child_text('role');
            if ($role eq 'Substitute') {
              $mp_data{NAME} =~ s/\|([^\]]+)\]/|''$1 (substitute)'']/;
            }
            elsif ($role ne 'Member') {
              $mp_data{NAME} =~ s/]]$/ ($role)]]/o;
            }

            $mp_data{TP} = $mp->first_child_text('score') || '-';
            $mp_data{AM138} = $am138{$mp->first_child_text('mepid')} || '-';

            push @meps, \%mp_data;
          }
        }
	  };

    if (my $e = catch std::exception) {
      warn "Query $query failed\n";
      warn $e->what() . "\n";
      exit( -1 );
    }
    elsif ($@) {
      warn "Query $query failed\n";
      warn $@;
      exit( -1 );
    }
};

if (my $e = catch std::exception) {
  warn "Query failed\n";
  warn $e->what() . "\n";
  exit( -1 );
}
elsif ($@) {
  warn "Query failed\n";
  warn $@;
  exit( -1 );
}

# Build mediawiki page
my $mediawiki_page = << "MEDIAWIKI_PAGE";
__NOEDITSECTION__
<!--<div class="LanguageLinks"><table width="100%"><tr valign="top" style="background: #EEF3E2"><td style="width: 25px; padding-left: 0.5em;">[[Image:Geographylogo.png|25px|Languages]]</td><td style="padding: 1px 1em 0; background: #F6F9ED;">'''[[MEPs_ITRE_am138|English]]''' - '''[[Eurodeputes_ITRE_am138|Français]]'''</td></tr></table></div>-->

==[[Political_Memory|Political Memory]]: ITRE MEPs' Informations with regard to amendment 138 in Telecoms Package 1st reading==

Find below informations about Members of European Parliament from the Committee on Industry (ITRE) and their vote on amendement 138 in first reading (September 24 2008) and thier global score for Telecoms Packages during this first reading.

MEDIAWIKI_PAGE

$mediawiki_page .= "{| border='1' cellpadding='5' cellspacing='0' class='wikitable sortable'\n";
$mediawiki_page .= "! MEP !! Am. 138 Vote !! Telecoms Package Score !! Country !! Group !! Contact !! Offices\n";
$mediawiki_page .= "|-\n";
foreach my $deputy (@meps) {
  (my $grp_wiki = $deputy->{GRP}) =~ s/[\/-].+$//o;
  my $am138_colour = $deputy->{AM138} eq 'for' ? '#00ff00' : $deputy->{AM138} eq 'against' ? '#ff0000' : $deputy->{AM138} eq 'abstention' ? '#dc143c' : '#ffffff';
  my $tp_colour = $deputy->{TP} ne '-' ? $colours[sprintf("%.0f", $deputy->{TP}/5)] : '#ffffff';
  $mediawiki_page .= "| $deputy->{NAME} || style=\"background:$am138_colour\" | $deputy->{AM138} || style=\"background:$tp_colour\" | $deputy->{TP} || $deputy->{COUNTRY} || {{#icon:$grp_wiki.png|$deputy->{GRP}||MEPs_$grp_wiki}} [[MEPs_$grp_wiki|$deputy->{GRP}]] || $deputy->{TEL}<br />$deputy->{FAX}<br />$deputy->{EMAIL} || Brussels: $deputy->{BXL}<br />Strasbourg: $deputy->{STR}\n";
  $mediawiki_page .= "|-\n";
}
$mediawiki_page =~ s/\|-\n$/|}\n/so;
$mediawiki_page .= "[[Category:Telecoms_Package]] [[Category:MEPs]] [[Category:Political_Memory]]\n";

# Output mediawiki page
my $mediawiki_location = "$output_dir/MEPs_ITRE_am138.mediawiki";
open MEDIAWIKI, ">$mediawiki_location" or die "Cannot write in file $mediawiki_location: $!\n";
print MEDIAWIKI $mediawiki_page;
close MEDIAWIKI or die "Error when closing $mediawiki_location: $!\n";
warn "$mediawiki_location done\n";
