#!/usr/bin/perl

# Copyright 2007-2008 Gérald Sédrati-Dinet, gibus@april.org
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, see <http://www.gnu.org/licenses/>.

use strict;
use warnings;
use utf8;
use open ':utf8';

use Sleepycat::DbXml 'simple';
use XML::Twig;
use Getopt::Long;

# Some files will be save under tree hierarchy based on executable location
use FindBin qw($Bin);
use lib "$Bin";
(my $root_dir = $Bin) =~ s/\/[^\/]*$//o;

# Default values
my $path2DbEnv = "$root_dir/dbxml";
my $theContainer = 'meps.dbxml';
my $theVoteContainer = 'votes.dbxml';
my $output_dir = "$root_dir/mediawiki/meps_groups";

# Parse command line options
Getopt::Long::Configure("bundling");
GetOptions(
  'path2dbenv|p=s'      => \$path2DbEnv,
  'container|c=s'       => \$theContainer,
  'vote_container|v=s'  => \$theVoteContainer,
  'output|o=s'          => \$output_dir,
  'help|h'              =>
    sub {
      print STDERR <<USAGE;
Usage: $0 [-h] [-p <path_to_db_env>] [-c <container>] [-o <output_dir]
Options can be:
  --path2dbenv, -p <path_to_db_env>: path to directory of DB XML database
                                     default: $root_dir/dbxml
  --container, -c <container>:       DB XML container
                                     <container> is typically meps (for convenience the final "s" can be omitted, also ".dbxml" can be appended)
                                     default: meps.dbxml
  --vote_container, -c <container>:  DB XML container for votes
                                     <vote_container> is typically votes (for convenience the final "s" can be omitted, also ".dbxml" can be appended)
                                     default: votes.dbxml
  --output, -o <output_dir>:         output directory of mediawiki pages
                                     default: $root_dir/mediawiki/meps_groups
  --help, -h:                        print this message
USAGE
      exit 0;
    }
);
$theContainer =~ s/s?(?:\.dbxml)?$/s/;
$theVoteContainer =~ s/s?(?:\.dbxml)?$/s/;

my %groups_order = (
  'PPE'       => 1,
  'SD'        => 2,
  'ALDE'      => 3,
  'Verts/ALE' => 5,
  'ECR'       => 4,
  'GUE/NGL'   => 6,
  'EFD'       => 7,
  'NI'        => 8,
);

my %meps;

eval {
    # Open containers in the db environment
    my $env = new DbEnv(0);
    $env->set_cachesize(0, 64 * 1024, 1);
    $env->open($path2DbEnv,
                Db::DB_INIT_MPOOL|Db::DB_CREATE|Db::DB_INIT_LOCK|Db::DB_INIT_LOG);
    my $theMgr = new XmlManager($env);

    my $container = $theMgr->openContainer("$theContainer.dbxml");
    my $vote_container = $theMgr->openContainer("$theVoteContainer.dbxml");
    my %am138;

    # Get Telecoms Package 1st reading vote
    my $vote_document = $vote_container->getDocument('Directives_Paquet_telecom_1re_lecture');
    my $vote_doc_string = $vote_document->getContent();
    my $vote_xml = XML::Twig->new(output_encoding=>'UTF-8',pretty_print => 'indented')->parse($vote_doc_string);
    foreach my $vote ($vote_xml->root->children('vote')) {
        next unless $vote->first_child('subject')->first_child_text('part') eq 'am. 138/1';
        foreach my $mep ($vote->first_child('result')->children('mep')) {
          if ($mep->first_child_text('choice') eq 'for') {
            $am138{$mep->first_child_text('dbxmlid')} = $mep->first_child_text('name');
          }
        }
    }

    # Get Telecoms Package 2nd reading vote
    $vote_document = $vote_container->getDocument('Directives_Paquet_telecom_2e_lecture');
    $vote_doc_string = $vote_document->getContent();
    $vote_xml = XML::Twig->new(output_encoding=>'UTF-8',pretty_print => 'indented')->parse($vote_doc_string);
    foreach my $vote ($vote_xml->root->children('vote')) {
        next unless $vote->first_child('subject')->first_child_text('part') eq 'ams. 1 art. 8.4.h=2=5=6=9';
        foreach my $mep ($vote->first_child('result')->children('mep')) {
          # Keep only meps who have voted for am138 in 1st.
          if (not exists $am138{$mep->first_child_text('dbxmlid')}) {
            next;
          }
          # Keep only meps who have voted for am138 in 2nd reading.
          elsif ($mep->first_child_text('choice') ne 'for') {
            delete $am138{$mep->first_child_text('dbxmlid')};
          }
        }
    }

    # Query for MEPs order by group
    my $query = <<"QUERY";
for \$group in distinct-values(collection("dbxml:/$theContainer.dbxml")/politician/infos/group/abbreviation)
return
  <result>
    <code>{\$group}</code>
    {
    for \$deputy in collection("dbxml:/$theContainer.dbxml")/politician[container/\@current/string()="true" and infos/group/abbreviation/string()=\$group]
    order by \$deputy/infos/constituency/country/code/string(), upper-case(\$deputy/infos/name/last/string()), upper-case(\$deputy/infos/name/first/string())
    return
      <deputy>
        <group>{ \$deputy/infos/group/name/string() }</group>
        <dbxmlid>{ \$deputy/infos/name/wiki/string() }</dbxmlid>
        <name>{ concat("[[", \$deputy/infos/name/wiki/string(), "/fr|", \$deputy/infos/name/full/string(), "]]") }</name>
        <country>{ concat("{{#icon:", \$deputy/infos/constituency/country/code/string(), ".png|", \$deputy/infos/constituency/country/name/string(), "||Eurodeputes_", \$deputy/infos/constituency/country/code/string(), "}} [[Eurodeputes_", \$deputy/infos/constituency/country/code/string(), "|", \$deputy/infos/constituency/country/name/string(), "]]") }</country>
        <tel>{ string-join(for \$tel in \$deputy/contact//phone return concat("Tel: [callto://", replace(\$tel/string(), " ", ""), " ", \$tel/string(), "]"), "<br />") }</tel>
        <fax>{ string-join(for \$fax in \$deputy/contact//fax return concat("Fax: ", \$fax/string()), "<br />") }</fax>
        <email>{ string-join(for \$email in \$deputy/contact/email return concat("[mailto:", \$email/string(), " ", replace(\$email/string(), "@", " (at)"), "]"), "<br />") }</email>
        <bxl>{ \$deputy/contact/address[label="Parlement européen"][city="Bruxelles"] }</bxl>
        <str>{ \$deputy/contact/address[label="Parlement européen"][city="Strasbourg Cedex"] }</str>
      </deputy>
    }
  </result>
QUERY
    #  Perform the query
    eval {
        my $results = $theMgr->query($query);
        my $value;
        while( $results->next($value) ) {
          my $xml = XML::Twig->new(output_encoding => 'UTF-8')->parse($value);
          my $code =  $xml->root->first_child_text('code');
          foreach my $mep ($xml->root->children('deputy')) {
            # Keep only meps who have voted for am138 in 1st AND 2nd reading.
            next unless exists $am138{$mep->first_child_text('dbxmlid')};

            $meps{$code}->{NAME} = $mep->first_child_text('group');
            my %mep_data;
            $mep_data{NAME} = $mep->first_child_text('name');
            $mep_data{COUNTRY} = $mep->first_child_text('country');
            $mep_data{GRP} = $mep->first_child_text('grp');
            $mep_data{TEL} = $mep->first_child_text('tel');
            $mep_data{TEL} =~ s/Tel: \[callto:\/\/\+32/Tel Bxl.: [callto:\/\/+32/go;
            $mep_data{TEL} =~ s/Tel: \[callto:\/\/\+33/Tel Str.: [callto:\/\/+33/go;
            $mep_data{FAX} = $mep->first_child_text('fax');
            $mep_data{FAX} =~ s/Fax: \+32/Fax Bxl.: +32/go;
            $mep_data{FAX} =~ s/Fax: \+33/Fax Str.: +33/go;
            $mep_data{EMAIL} = $mep->first_child_text('email');
            my $bxl_addr = $mep->first_child('bxl')->first_child('address');
            $mep_data{BXL} = $bxl_addr->first_child('building')->first_child_text('abbreviation') . ' ' . $bxl_addr->first_child_text('office');
            if ($mep_data{BXL} eq '- Moving') {
              $mep_data{BXL} = '[[Eurodeputes_BXL_SDF|-]]';
            }
            else {
              $mep_data{BXL} =~ s/^(([A-Z][A-Z][A-Z]) (\d\d).+)$/[[Eurodeputes_$2_$3|$1]]/o;
            }

            my $str_addr = $mep->first_child('str')->first_child('address');
            $mep_data{STR} = $str_addr->first_child('building')->first_child_text('abbreviation') . ' ' . $str_addr->first_child_text('office');
            if ($mep_data{STR} eq '- Moving') {
              $mep_data{STR} = '[[Eurodeputes_STR_SDF|-]]';
            }
            else {
              $mep_data{STR} =~ s/^(([A-Z][A-Z][A-Z]) [MT](\d\d).+)$/[[Eurodeputes_$2_$3|$1]]/o;
            }

            push @{$meps{$code}->{MEPS}}, \%mep_data;
          }
        }
	  };

    if (my $e = catch std::exception) {
      warn "Query $query failed\n";
      warn $e->what() . "\n";
      exit( -1 );
    }
    elsif ($@) {
      warn "Query $query failed\n";
      warn $@;
      exit( -1 );
    }
};

if (my $e = catch std::exception) {
  warn "Query failed\n";
  warn $e->what() . "\n";
  exit( -1 );
}
elsif ($@) {
  warn "Query failed\n";
  warn $@;
  exit( -1 );
}

# Build mediawiki page
my $mediawiki_page = << "MEDIAWIKI_PAGE";
__NOEDITSECTION__
<div class="LanguageLinks"><table width="100%"><tr valign="top" style="background: #EEF3E2"><td style="width: 25px; padding-left: 0.5em;">[[Image:Geographylogo.png|25px|Languages]]</td><td style="padding: 1px 1em 0; background: #F6F9ED;">'''[[MEPs_am138_1st_2nd_readings|English]]''' - '''[[Eurodeputes_am138_1re_2e_lectures|Français]]'''</td></tr></table></div>

==[[Political_Memory|Political Memory]]: Informations About MEPs Who Have Voted For Am138 In 1st and 2nd Readings of Telecoms Package==

MEDIAWIKI_PAGE

foreach my $group_code (sort {$groups_order{$a} <=> $groups_order{$b}} keys %meps) {
  next unless exists($meps{$group_code}->{MEPS});

  (my $group_wiki = $group_code) =~ s/[\/-].+$//o;
  $mediawiki_page .= "\n===[[Eurodeputes_$group_wiki|$meps{$group_code}->{NAME}]]===\n\n";
  $mediawiki_page .= "{| border='1' cellpadding='5' cellspacing='0' width='90%'\n";
  $mediawiki_page .= "! Eurodéputé !! Pays !! Contact !! Bureaux\n";
  $mediawiki_page .= "|-\n";
  foreach my $deputy (@{$meps{$group_code}->{MEPS}}) {
    $mediawiki_page .= "| $deputy->{NAME} || $deputy->{COUNTRY} || $deputy->{TEL}<br />$deputy->{FAX}<br />$deputy->{EMAIL} || Bruxelles: $deputy->{BXL}<br />Strasbourg: $deputy->{STR}\n";
    $mediawiki_page .= "|-\n";
  }
  $mediawiki_page =~ s/\|-\n$/|}\n/so;
}

$mediawiki_page .= "\n\n----\n\nRetour à la page d'accueil de [[Memoire_politique|Mémoire politique]]\n";
$mediawiki_page .= "[[Category:Telecoms_Package]] [[Category:Eurodéputés]] [[Category:Mémoire_Politique]]\n";

# Output mediawiki page
my $mediawiki_location = "$output_dir/Eurodeputes_am138_1re_2e_lectures.mediawiki";
open MEDIAWIKI, ">$mediawiki_location" or die "Cannot write in file $mediawiki_location: $!\n";
print MEDIAWIKI $mediawiki_page;
close MEDIAWIKI or die "Error when closing $mediawiki_location: $!\n";
warn "$mediawiki_location done\n";
