#!/usr/bin/perl

# Copyright 2007-2008 Gérald Sédrati-Dinet, gibus@april.org
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, see <http://www.gnu.org/licenses/>.

use strict;
use warnings;

use XML::Twig;
use Getopt::Long;
use File::Basename;
use Sleepycat::DbXml 'simple';
use POSIX 'locale_h';
use Unicode::String qw(utf8 latin1);

# Some files will be save under tree hierarchy based on executable location
use FindBin qw($Bin);
use lib "$Bin";
(my $root_dir = $Bin) =~ s/\/[^\/]*$//o;

# Default values
my $path2DbEnv = "$root_dir/dbxml";
my $theContainer = 'mps.dbxml';
my $output_dir = "$root_dir/xml/mps_votes";

# Parse command line options
Getopt::Long::Configure("bundling");
GetOptions(
  'output|o=s'     => \$output_dir,
  'container|c=s'  => \$theContainer,
  'help|h'         =>
    sub {
      print STDERR <<USAGE;
Usage: $0 [-h] [-p <path_to_db_env>] [-c <container>] [-o <output_dir] <input_text_file>
Options can be:
  --path2dbenv, -p <path_to_db_env>: path to directory of DB XML database
                                     default: $root_dir/dbxml
  --container, -c <container>:       DB XML container
                                     <container> is typically mps (for convenience the final "s" can be omitted, also ".dbxml" can be appended)
                                     default: mps.dbxml
  --output, -o <output_dir>:         output directory
                                     default: $root_dir/xml/mps_votes
  --help, -h:                        print this message
USAGE
      exit 0;
    }
);

$theContainer =~ s/s?(?:\.dbxml)?$/s/;

# Open a container in the db environment
my $env = new DbEnv(0);
$env->set_cachesize(0, 64 * 1024, 1);
$env->open($path2DbEnv,
            Db::DB_INIT_MPOOL|Db::DB_CREATE|Db::DB_INIT_LOCK|Db::DB_INIT_LOG);
my $theMgr = new XmlManager($env);

my $container = $theMgr->openContainer("$theContainer.dbxml");

my $input_file = shift or die "Usage : $0 [-h] [-p <path_to_db_env>] [-c <container>] [-o <output_dir] <input_text_file>\n";
my $file_name = fileparse($input_file, qr/\Q.txt\E/);

# Read input file
open INPUT_FILE, $input_file or die "Cannot open $input_file: $!\n";
my @input_lines = <INPUT_FILE>;
close INPUT_FILE;

my @votes     = (); # List of votes
my %this_vote = (); # Structure to collect current vote
my $status    = ''; # Flag to indicate which part is currently parsed
my $voters    = ''; # Concatenated string of voters (needed since a MP name can be splitted on continuous lines)
my %memoize   = (); # Hash to memoize relation between voting names and MP identifiers in dbxml

# Parse input file
foreach my $line (@input_lines) {
  chomp $line;

  # Skip group
  next if $line =~ /^(?:Groupe|Députés n'appartenant à aucun groupe)/o;
  # Trim line
  $line =~ s/^\s+//o;
  $line =~ s/\s+$//o;
  $line =~ s/\.$//o;
  # Delete M. and like
  $line =~ s/(?:MM\.|M\.|Mmes|Mme) //go;
  # Replace et
  $line =~ s/ et /, /go;

  # Beginning of a new vote
  if ($line =~ /^\s*Analyse du scrutin n° \d+ - Séance du (?:: )?(\d\d)\/(\d\d)\/(\d\d\d\d) - Scrutin public sur (?:l'|la |le )(.+?) (?:au|du|à la|de la) ((?:projet|proposition) de loi(?:, adoptée? par le Sénat,?)?(?:après engagement de la procédure accélérée,)? .+)$/o) {
    # End of previous vote (case where no correction of votes have happened)
    if ($status eq 'abstention') {
      $voters =~ s/^, //go;
      $voters =~ s/ ,/,/go;
      $voters =~ s/ $//go;
      push @{$this_vote{VOTE}->{ABSTENTION}}, split(/, /, $voters);
      $voters = '';

      my %vote = %this_vote;
      push @votes, \%vote;
    }

    %this_vote = (
                  SUBJECT => {
                              TEXT       => $5,
                              PART       => $4,
                             },
                  DATE    => {
                              DAY        => $1,
                              MONTH      => $2,
                              YEAR       => $3,
                             },
                  VOTE    => {
                              FOR        => [],
                              AGAINST    => [],
                              ABSTENTION => [],
                             },
                 );
    $status = 'beginning';
  }

  # Beginning of FOR votes
  elsif ($status eq 'beginning' and ($line =~ /^Pour ?: \d+$/o)) {
    $status = 'for';
  }

  # Collect FOR votes
  elsif ($status eq 'for') {
    # End of FOR votes, beginning of AGAINST votes
    if ($line =~ /^Contre ?: \d+$/o) {
      $status = 'against';
      $voters =~ s/^,\s+//go;
      $voters =~ s/\s+,/,/go;
      $voters =~ s/\s+$//go;
      push @{$this_vote{VOTE}->{FOR}}, split(/, /, $voters);
      $voters = '';
    }
    else {
      $line =~ s/$/ /o unless $line =~ /[ -]$/o;
      $voters .= $line;
    }
  }

  # Collect AGAINST votes
  elsif ($status eq 'against') {
    # End of AGAINST votes, beginning of ABSTENTION votes
    if ($line =~ /^Abstention ?: \d+$/o) {
      $status = 'abstention';
      $voters =~ s/^,\s+//go;
      $voters =~ s/\s+,/,/go;
      $voters =~ s/\s+$//go;
      push @{$this_vote{VOTE}->{AGAINST}}, split(/, /, $voters);
      $voters = '';
    }
    else {
      $line =~ s/$/ /o unless $line =~ /[ -]$/o;
      $voters .= $line;
    }
  }

  # Collect ABSTENTION votes
  elsif ($status eq 'abstention') {
    # End of ABSTENSION votes, waiting for beginning of a new vote
    if ($line =~ /^(?:Non-votant|MISES AU POINT AU SUJET DU PRESENT SCRUTIN)/o) {
      $status = '';
      $voters =~ s/^,\s+//go;
      $voters =~ s/\s+,/,/go;
      $voters =~ s/\s+$//go;
      push @{$this_vote{VOTE}->{ABSTENTION}}, split(/, /, $voters);
      $voters = '';

      my %vote = %this_vote;
      push @votes, \%vote;
    }
    else {
      $line =~ s/$/ /o unless $line =~ /[ -]$/o;
      $voters .= $line;
    }
  }
}

# Collect the last vote
if ($status eq 'abstention') {
  $voters =~ s/^,\s+//go;
  $voters =~ s/\s+,/,/go;
  $voters =~ s/\s+$//go;
  push @{$this_vote{VOTE}->{ABSTENTION}}, split(/, /, $voters);
  push @votes, \%this_vote;
}

# Build XML tree
my $xml = XML::Twig->new(pretty_print => 'indented');
$xml->set_encoding('UTF-8');
my $root = XML::Twig::Elt->new('votes');
$xml->set_root($root);
$root->insert_new_elt('first_child', 'container', 'mps_votes.dbxml');

foreach my $vote (@votes) {
  my $vote_elt = $root->insert_new_elt('last_child', 'vote');

  my $subject = $vote_elt->insert_new_elt('last_child', 'subject');
  $subject->insert_new_elt('last_child', 'text', $vote->{SUBJECT}->{TEXT});
  $subject->insert_new_elt('last_child', 'part', $vote->{SUBJECT}->{PART});

  my $date = $vote_elt->insert_new_elt('last_child', 'date');
  $date->insert_new_elt('last_child', 'year', $vote->{DATE}->{YEAR});
  $date->insert_new_elt('last_child', 'month', $vote->{DATE}->{MONTH});
  $date->insert_new_elt('last_child', 'day', $vote->{DATE}->{DAY});

  my $result = $vote_elt->insert_new_elt('last_child', 'result');
  foreach my $mp (@{$vote->{VOTE}->{FOR}}) {
    # Search for MP's name in dbxml
    my ($name, $wiki_name) = mp_name($mp);
    $name = $mp unless $name;
    my $mp_elt = $result->insert_new_elt('last_child', 'mp');
    $mp_elt->insert_new_elt('last_child', 'name', $name);
    $mp_elt->insert_new_elt('last_child', 'dbxmlid', $wiki_name) if $wiki_name;
    $mp_elt->insert_new_elt('last_child', 'choice', 'for');
  }
  foreach my $mp (@{$vote->{VOTE}->{AGAINST}}) {
    # Search for MP's name in dbxml
    my ($name, $wiki_name) = mp_name($mp);
    $name = $mp unless $name;
    my $mp_elt = $result->insert_new_elt('last_child', 'mp');
    $mp_elt->insert_new_elt('last_child', 'name', $name);
    $mp_elt->insert_new_elt('last_child', 'dbxmlid', $wiki_name) if $wiki_name;
    $mp_elt->insert_new_elt('last_child', 'choice', 'against');
  }
  foreach my $mp (@{$vote->{VOTE}->{ABSTENTION}}) {
    # Search for MP's name in dbxml
    my ($name, $wiki_name) = mp_name($mp);
    $name = $mp unless $name;
    my $mp_elt = $result->insert_new_elt('last_child', 'mp');
    $mp_elt->insert_new_elt('last_child', 'name', $name);
    $mp_elt->insert_new_elt('last_child', 'dbxmlid', $wiki_name) if $wiki_name;
    $mp_elt->insert_new_elt('last_child', 'choice', 'abstention');
  }
}

my $xml_location = "$output_dir/$file_name.xml";
open XML, ">$xml_location" or die "Cannot write in file $xml_location: $!\n";
$xml->flush(\*XML);
close XML or die "Error when closing $xml_location: $!\n";

sub mp_name {
  my $voting_name = shift;

  # Memoization
  if (exists $memoize{$voting_name}) {
    return wantarray ? ($memoize{$voting_name}->[0], $memoize{$voting_name}->[1]) : $memoize{$voting_name}->[1];
  }
  my $memoize_name = $voting_name;

  $voting_name = wikify($voting_name);

  my $theDocument;
  eval {
      $theDocument = $container->getDocument($voting_name);
  };
  if ($@) {
    warn "Not found voting_name=<$memoize_name> searched=<$voting_name>\n";
    $memoize{$memoize_name} = [$memoize_name, undef];
    return wantarray ? ($memoize{$memoize_name}->[0], $memoize{$memoize_name}->[1]) : $memoize{$memoize_name}->[1];
  }

  # Eventually fix voting name
  my $xml = XML::Twig->new(keep_encoding=>1)->parse($theDocument->getContent());
  my $name = $xml->root->first_child('infos')->first_child('name');
  my $real_name = $name->first_child_text('first') . ' ' . $name->first_child_text('last');
  
  $memoize{$memoize_name} = [$real_name, $voting_name];

  return wantarray ? ($memoize{$memoize_name}->[0], $memoize{$memoize_name}->[1]) : $memoize{$memoize_name}->[1];
}

sub wikify {
  my $str = shift;
  $str = desaccent($str);
  $str =~ s/-//go;
  return $str;
}

sub desaccent {
  my $str = shift;
  my $mp_name = $str;
  $str =~ s/À/A/go;
  $str =~ s/Â/A/go;
  $str =~ s/Ä/A/go;
  $str =~ s/É/E/go;
  $str =~ s/È/E/go;
  $str =~ s/Ê/E/go;
  $str =~ s/Ë/E/go;
  $str =~ s/Í/I/go;
  $str =~ s/Î/I/go;
  $str =~ s/Ï/I/go;
  $str =~ s/Ó/O/go;
  $str =~ s/Ô/O/go;
  $str =~ s/Ö/O/go;
  $str =~ s/Ù/U/go;
  $str =~ s/Û/U/go;
  $str =~ s/Ü/U/go;
  $str =~ s/Ç/C/go;
  $str =~ s/à/a/go;
  $str =~ s/â/a/go;
  $str =~ s/ä/a/go;
  $str =~ s/é/e/go;
  $str =~ s/è/e/go;
  $str =~ s/ê/e/go;
  $str =~ s/ë/e/go;
  $str =~ s/í/i/go;
  $str =~ s/î/i/go;
  $str =~ s/ï/i/go;
  $str =~ s/ó/o/go;
  $str =~ s/ô/o/go;
  $str =~ s/ö/o/go;
  $str =~ s/ù/u/go;
  $str =~ s/û/u/go;
  $str =~ s/ü/u/go;
  $str =~ s/ç/c/go;
  $str =~ s/[ ']//go;
  return $str;
}
