#!/usr/bin/perl

# Copyright 2007-2008 Gérald Sédrati-Dinet, gibus@april.org
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, see <http://www.gnu.org/licenses/>.

use strict;
use warnings;

use XML::Twig;
use Getopt::Long;
use File::Basename;
use Sleepycat::DbXml 'simple';
use POSIX 'locale_h';
use Unicode::String qw(utf8 latin1);

# Some files will be save under tree hierarchy based on executable location
use FindBin qw($Bin);
use lib "$Bin";
(my $root_dir = $Bin) =~ s/\/[^\/]*$//o;

# Default values
my $path2DbEnv = "$root_dir/dbxml";
my $theContainer = 'mps.dbxml';
my $output_dir = "$root_dir/xml/mps_votes";

# Parse command line options
sub usage {
    print STDERR <<USAGE;
Usage: $0 [-h] [-p <path_to_db_env>] <group> <vote>
Options can be:
  <group>:                           Name of the political group to process
                                     (UMP, SRC, GDR, NC, NI)
  <vote>:                            Vote for the group (for, against, abstention)
  --path2dbenv, -p <path_to_db_env>: path to directory of DB XML database
                                     default: $root_dir/dbxml
  --help, -h:                        print this message
USAGE
      exit 0;
}

Getopt::Long::Configure("bundling");
GetOptions(
  'path2dbenv|p=s' => \$path2DbEnv,
  'help|h'         => \&usage,
);

$theContainer =~ s/s?(?:\.dbxml)?$/s/;

my $group = shift or usage();
if ($group ne 'UMP' and
    $group ne 'SRC' and
    $group ne 'GDR' and
    $group ne 'NC' and
    $group ne 'NI') {
        die "$group is not a known group: it has to be either UMP or SRC or GDR or NC or NI.\n";
}
my $group_condition = $group ne 'NI' ?
    "contains(infos/group/abbreviation/string(),\"$group\")" :
    "infos/group/name/string()=\"Députés n'appartenant à aucun groupe\"";

my $vote = shift or usage();
if ($vote ne 'for' and
    $vote ne 'against' and
    $vote ne 'abstention') {
        die "$vote is not a known vote: it has to be either for or against or abstention.\n";
}

eval {
    # Open a container in the db environment
    my $env = new DbEnv(0);
    $env->set_cachesize(0, 64 * 1024, 1);
    $env->open($path2DbEnv,
                Db::DB_INIT_MPOOL|Db::DB_CREATE|Db::DB_INIT_LOCK|Db::DB_INIT_LOG);
    my $theMgr = new XmlManager($env);

    my $container = $theMgr->openContainer("$theContainer.dbxml");

    my $query = <<"QUERY";
      for \$deputy in collection("dbxml:/$theContainer.dbxml")/politician[container/\@current/string()="true"][$group_condition]
return
      <mp>
        <name>{ concat(\$deputy/infos/name/first/string(), " ", \$deputy/infos/name/last/string()) }</name>
        <dbxmlid>{ \$deputy/infos/name/wiki/string() }</dbxmlid>
        <choice>$vote</choice>
      </mp>
QUERY
    #  Perform the query
    eval {
      my $results = $theMgr->query($query);
      my $value;
      while( $results->next($value) ) {
        $value =~ s/></>\n        </go;
        $value =~ s/        <\/mp>/      <\/mp>/o;
        print "      $value\n";
      }
	};

    if (my $e = catch std::exception) {
      warn "Query $query failed\n";
      warn $e->what() . "\n";
      exit( -1 );
    }
    elsif ($@) {
      warn "Query $query failed\n";
      warn $@;
      exit( -1 );
    }
};

if (my $e = catch std::exception) {
  warn "Query failed\n";
  warn $e->what() . "\n";
  exit( -1 );
}
elsif ($@) {
  warn "Query failed\n";
  warn $@;
  exit( -1 );
}
