#!/usr/bin/perl

# Copyright 2007-2008 Gérald Sédrati-Dinet, gibus@april.org
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, see <http://www.gnu.org/licenses/>.

use strict;
use warnings;

use XML::Twig;
use Getopt::Long;
use File::Basename;
use Sleepycat::DbXml 'simple';
use POSIX 'locale_h';
use Unicode::String qw(utf8 latin1);
use LWP::UserAgent;

# Some files will be save under tree hierarchy based on executable location
use FindBin qw($Bin);
use lib "$Bin";
(my $root_dir = $Bin) =~ s/\/[^\/]*$//o;

# Default values
my $path2DbEnv = "$root_dir/dbxml";
my $theContainer = 'mps.dbxml';
my $output_dir = "$root_dir/xml/mps_votes";
my $url_scores = "http://www.laquadrature.net/wiki/Loi_Hadopi_Assemblee_nationale_interventions_en_seance";

# Parse command line options
sub usage {
    print STDERR <<USAGE;
Usage: $0 [-h] [-p <path_to_db_env>] [-u <url_scores_debates>]
Options can be:
  --url, -u <url_scores_debates>:    url of the wiki page where debates are scored
                                     default: http://www.laquadrature.net/wiki/Loi_Hadopi_Assemblee_nationale_interventions_en_seance
  --path2dbenv, -p <path_to_db_env>: path to directory of DB XML database
                                     default: $root_dir/dbxml
  --help, -h:                        print this message
USAGE
      exit 0;
}

Getopt::Long::Configure("bundling");
GetOptions(
  'path2dbenv|p=s' => \$path2DbEnv,
  'url|u=s'        => \$url_scores,
  'help|h'         => \&usage,
);

$theContainer =~ s/s?(?:\.dbxml)?$/s/;

# Create a user agent object
my $ua = LWP::UserAgent->new(env_proxy => 1);
$ua->agent("$0 (gibus perl script to fetch information from http://www.laquadrature.net/wiki/)");
my $headers = HTTP::Headers->new('Accept-Language' => 'fr');

# Fetch the wiki page
my $wiki_req = HTTP::Request->new(GET => $url_scores, $headers);
my $wiki_res = $ua->request($wiki_req);
unless ($wiki_res->is_success) {
  die "Error fetching wiki page: $url_scores: ", $wiki_res->status_line, "\n";
}
my $wiki_html = $wiki_res->content;
die "Unknown wiki page $url_scores\n" if $wiki_html =~ /<div class="noarticletext">/o;

# Extract opinions
my %mp_scores;
foreach my $wiki_line (split /\n/, $wiki_html) {
    next unless $wiki_line =~ /&nbsp;: \d+\/20/o;
    my @scores = ($wiki_line =~ /(\d+)\/20/g);
    (my $mp) = ($wiki_line =~ /<a href="\/wiki\/([^#]+)#/);
    die "<$wiki_line>\n" unless $mp;
    if (exists $mp_scores{$mp}) {
        push @{$mp_scores{$mp}}, @scores;
    }
    else {
        $mp_scores{$mp} = \@scores;
    }
}

print <<HEADER;
  <vote>
    <subject>
      <text>Projet de loi, adopté, par le Sénat, favorisant la diffusion et la protection de la création sur internet, n° 1240</text>
      <part>Interventions en séance</part>
      <description>[[Loi_Hadopi_Assemblee_nationale_interventions_en_seance#Intervenants|Interventions en séance]], notées sur 20.</description>
      <weight>50</weight>
    </subject>
    <date>
      <year>2009</year>
      <month>11</month>
      <day>03</day>
    </date>
    <result>
HEADER

eval {
    # Open a container in the db environment
    my $env = new DbEnv(0);
    $env->set_cachesize(0, 64 * 1024, 1);
    $env->open($path2DbEnv,
                Db::DB_INIT_MPOOL|Db::DB_CREATE|Db::DB_INIT_LOCK|Db::DB_INIT_LOG);
    my $theMgr = new XmlManager($env);

    my $container = $theMgr->openContainer("$theContainer.dbxml");

    my $query = <<"QUERY";
      for \$deputy in collection("dbxml:/$theContainer.dbxml")/politician[container/\@current/string()="true"]
return
      <mp>
        <name>{ concat(\$deputy/infos/name/first/string(), " ", \$deputy/infos/name/last/string()) }</name>
        <dbxmlid>{ \$deputy/infos/name/wiki/string() }</dbxmlid>
        <choice>8</choice>
      </mp>
QUERY
    #  Perform the query
    eval {
      my $results = $theMgr->query($query);
      my $value;
      while( $results->next($value) ) {
        $value =~ s/></>\n        </go;
        $value =~ s/        <\/mp>/      <\/mp>/o;
        (my $mp) = ($value =~ /<dbxmlid>([^<]+)<\/dbxmlid>/o);
        if (exists $mp_scores{$mp}) {
            my $mean_score = 0;
            foreach (@{$mp_scores{$mp}}) {
                $mean_score += $_;
            }
            $mean_score /= scalar @{$mp_scores{$mp}};
            $mean_score = sprintf "%.2f", $mean_score;
            $value =~ s/<choice>8<\/choice>/<choice>$mean_score<\/choice>/o;
        }
        print "      $value\n";
      }
	};

    if (my $e = catch std::exception) {
      warn "Query $query failed\n";
      warn $e->what() . "\n";
      exit( -1 );
    }
    elsif ($@) {
      warn "Query $query failed\n";
      warn $@;
      exit( -1 );
    }
};

if (my $e = catch std::exception) {
  warn "Query failed\n";
  warn $e->what() . "\n";
  exit( -1 );
}
elsif ($@) {
  warn "Query failed\n";
  warn $@;
  exit( -1 );
}

print <<FOOTER;
    </result>
  </vote>
FOOTER
