#!/usr/bin/perl

# Copyright 2007-2008 Gérald Sédrati-Dinet, gibus@april.org
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, see <http://www.gnu.org/licenses/>.

use strict;
use warnings;
use utf8;
use open ':utf8';

use Sleepycat::DbXml 'simple';
use XML::Twig;
use Getopt::Long;

# Some files will be save under tree hierarchy based on executable location
use FindBin qw($Bin);
use lib "$Bin";
(my $root_dir = $Bin) =~ s/\/[^\/]*$//o;

# Default values
my $path2DbEnv = "$root_dir/dbxml";
my $theContainer = 'mps_votes.dbxml';
my $theMpContainer = 'mps.dbxml';
my $output_dir = "$root_dir/mediawiki/mps_votes";
my $detail = 0;

# Parse command line options
Getopt::Long::Configure("bundling");
GetOptions(
  'path2dbenv|p=s'    => \$path2DbEnv,
  'container|c=s'     => \$theContainer,
  'mp_container|m=s' => \$theMpContainer,
  'output|o=s'        => \$output_dir,
  'detail|d'          => \$detail,
  'help|h'            =>
    sub {
      print STDERR <<USAGE;
Usage: $0 [-hd] [-p <path_to_db_env>] [-c <container>] [-o <output_dir] <VoteWikiName>
Options can be:
  --path2dbenv, -p <path_to_db_env>: path to directory of DB XML database
                                     default: $root_dir/dbxml
  --container, -c <container>:       DB XML container for votes
                                     <container> is typically votes (for convenience the final "s" can be omitted, also ".dbxml" can be appended)
                                     default: votes.dbxml
  --mp_container, -m <container>:   DB XML container for MPs
                                     <container> is typically mps (for convenience the final "s" can be omitted, also ".dbxml" can be appended)
                                     default: mps.dbxml
  --output, -o <output_dir>:         output directory of mediawiki pages
                                     default: $root_dir/mediawiki/votes
  --detail, -d:                      output global result on a page and details on another page
  --help, -h:                        print this message
USAGE
      exit 0;
    }
);
$theContainer =~ s/s?(?:\.dbxml)?$/s/;
$theMpContainer =~ s/s?(?:\.dbxml)?$/s/;

my $detail_wiki = $detail ? '_details' : '';
my $detail_text = $detail ? ' détaillés' : '';

my $vote_wiki = shift or die "You should provide the wiki name of the vote, see $0 -h\n";

my %votes;
my %scores;
my @parts;
my $vote_label;

my @colours = (
  '#ff0000',
  '#ff1900',
  '#ff3300',
  '#ff4c00',
  '#ff6600',
  '#ff7f00',
  '#ff9900',
  '#ffb200',
  '#ffcc00',
  '#ffe500',
  '#ffff00',
  '#e5ff00',
  '#ccff00',
  '#b2ff00',
  '#99ff00',
  '#7fff00',
  '#66ff00',
  '#4cff00',
  '#33ff00',
  '#19ff00',
  '#00FF00',
);

my %categories = (
  Loi_Hadopi_Assemblee_nationale => '[[Category:HADOPI_fr]] [[Category:Lois_fr]] [[Category:Mémoire_Politique]]',
  Vote_final_loi_Hadopi => '[[Category:HADOPI_fr]] [[Category:Lois_fr]] [[Category:Mémoire_Politique]]',
  Loi_Hadopi_2_Assemblee_nationale => '[[Category:HADOPI_fr]] [[Category:Lois_fr]] [[Category:Mémoire_Politique]]',
);

my %department_preposition = (
    '01' => 'de l\'',
    '02' => 'de l\'',
    '03' => 'de l\'',
    '07' => 'de l\'',
    '09' => 'de l\'',
    '10' => 'de l\'',
    '11' => 'de l\'',
    '12' => 'de l\'',
    '27' => 'de l\'',
    '28' => 'de l\'',
    '34' => 'de l\'',
    '35' => 'de l\'',
    '36' => 'de l\'',
    '37' => 'de l\'',
    '38' => 'de l\'',
    '60' => 'de l\'',
    '61' => 'de l\'',
    '89' => 'de l\'',
    '91' => 'de l\'',
    '04' => 'des',
    '05' => 'des',
    '06' => 'des',
    '08' => 'des',
    '13' => 'des',
    '22' => 'des',
    '40' => 'des',
    '64' => 'des',
    '65' => 'des',
    '66' => 'des',
    '78' => 'des',
    '79' => 'des',
    '88' => 'des',
    '92' => 'des',
    '14' => 'du',
    '15' => 'du',
    '18' => 'du',
    '25' => 'du',
    '29' => 'du',
    '30' => 'du',
    '32' => 'du',
    '39' => 'du',
    '41' => 'du',
    '45' => 'du',
    '46' => 'du',
    '47' => 'du',
    '49' => 'du',
    '56' => 'du',
    '59' => 'du',
    '62' => 'du',
    '63' => 'du',
    '67' => 'du',
    '68' => 'du',
    '69' => 'du',
    '81' => 'du',
    '82' => 'du',
    '83' => 'du',
    '84' => 'du',
    '90' => 'du',
    '94' => 'du',
    '95' => 'du',
    '16' => 'de la',
    '17' => 'de la',
    '19' => 'de la',
    '2A' => 'de la',
    '2B' => 'de la',
    '21' => 'de la',
    '23' => 'de la',
    '24' => 'de la',
    '26' => 'de la',
    '31' => 'de la',
    '33' => 'de la',
    '42' => 'de la',
    '43' => 'de la',
    '44' => 'de la',
    '48' => 'de la',
    '50' => 'de la',
    '51' => 'de la',
    '52' => 'de la',
    '53' => 'de la',
    '54' => 'de la',
    '55' => 'de la',
    '57' => 'de la',
    '58' => 'de la',
    '70' => 'de la',
    '71' => 'de la',
    '72' => 'de la',
    '73' => 'de la',
    '74' => 'de la',
    '76' => 'de la',
    '77' => 'de la',
    '80' => 'de la',
    '85' => 'de la',
    '86' => 'de la',
    '87' => 'de la',
    '93' => 'de la',
    '971' => 'de la',
    '972' => 'de la',
    '973' => 'de la',
    '974' => 'de la',
    '987' => 'de la',
    '988' => 'de la',
    '75' => 'de',
    '975' => 'de',
    '976' => 'de',
    '986' => 'de',
);

# Open a container in the db environment
my $env = new DbEnv(0);
$env->set_cachesize(0, 64 * 1024, 1);
$env->open($path2DbEnv,
          Db::DB_INIT_MPOOL|Db::DB_CREATE|Db::DB_INIT_LOCK|Db::DB_INIT_LOG);
my $theMgr = new XmlManager($env);

my $container = $theMgr->openContainer("$theContainer.dbxml");
my $mp_container = $theMgr->openContainer("$theMpContainer.dbxml");

# Get the document
eval {
  my $theDocument = $container->getDocument($vote_wiki);
  my $docString = $theDocument->getContent();

  # Build xml tree
  my $xml = XML::Twig->new(output_encoding => 'UTF-8')->parse($docString);

  # Extract infos
  $vote_label = $xml->root->first_child_text('label');

  foreach my $vote_elt ($xml->root->children('vote')) {
    my $vote_subject = $vote_elt->first_child('subject');
    my $text = $vote_subject->first_child_text('text');
    if (exists $votes{TEXT}) {
      my $txt_regexp = $text;
      $txt_regexp =~ s/([\(\)\[\]\+\*\.\?])/\\$1/go;
      unless ($votes{TEXT} =~ /$txt_regexp/) {
        $votes{TEXT} .= " / $text";
      }
    }
    else {
      $votes{TEXT} = $text;
    }

    my $date_elt = $vote_elt->first_child('date');
    my $date = sprintf "%s/%s/%04d", $date_elt->first_child_text('day'), $date_elt->first_child_text('month'), $date_elt->first_child_text('year');
    if (exists $votes{DATE}) {
      unless ($votes{DATE} =~ /$date/) {
        $votes{DATE} .= " / $date";
      }
    }
    else {
      $votes{DATE} = $date;
    }

    my $part = $vote_subject->first_child_text('part');
    # Append text name in case of another text with the same part already exists
    $part .= "_$text" if $votes{VOTES}->{$part};

    $votes{VOTES}->{$part}->{DESCR} = $vote_subject->first_child_text('description');
    $votes{VOTES}->{$part}->{REC} = $vote_subject->first_child_text('recommendation');
    $votes{VOTES}->{$part}->{WEIGHT} = $vote_subject->first_child_text('weight');
    push @parts, $part;

    my $result_elt = $vote_elt->first_child('result');
    foreach my $mp ($result_elt->children('mp')) {
      my $name = $mp->first_child_text('name');
      unless (exists $scores{$name}) {
        if ($mp->first_child('dbxmlid')) {
          my $dbxmlid = $mp->first_child_text('dbxmlid');
          $scores{$name}->{DBXMLID} = $dbxmlid;

          # Get MP document
          my $mpDocument = $mp_container->getDocument($dbxmlid);
          my $mpString = $mpDocument->getContent();
          my $mp_xml = XML::Twig->new(output_encoding => 'UTF-8')->parse($mpString);

          my $mp_infos = $mp_xml->root->first_child('infos');
          $scores{$name}->{NAME}->{FIRST} = $mp_infos->first_child('name')->first_child_text('first');
          $scores{$name}->{NAME}->{LAST} = $mp_infos->first_child('name')->first_child_text('last');
          $scores{$name}->{DEPARTMENT}->{NUMBER} = $mp_infos->first_child('constituency')->first_child('department')->first_child_text('number');
          $scores{$name}->{DEPARTMENT}->{NAME} = $mp_infos->first_child('constituency')->first_child('department')->first_child_text('name');
          $scores{$name}->{GRP}->{CODE} = $mp_infos->first_child('group')->first_child_text('abbreviation');
          $scores{$name}->{GRP}->{CODE} =~ s/Ap\. //o;
          $scores{$name}->{GRP}->{NAME} = $mp_infos->first_child('group')->first_child_text('name');
          $scores{$name}->{GRP}->{CODE} = 'NI' 
            if ($scores{$name}->{GRP}->{NAME} eq "Députés n'appartenant à aucun groupe");
        }
      }
      $scores{$name}->{VOTES}->{$part}->{VOTE} = $mp->first_child_text('choice');
      
      # Compute score for this vote
      if ($part ne 'Interventions en séance') {
      # unless there is no recommendation
        next unless $votes{VOTES}->{$part}->{REC};
        if ($votes{VOTES}->{$part}->{REC} eq 'for') {
          if ($scores{$name}->{VOTES}->{$part}->{VOTE} eq 'for') {
            $scores{$name}->{VOTES}->{$part}->{SCORE} = 2;
          }
          elsif ($scores{$name}->{VOTES}->{$part}->{VOTE} eq 'against') {
            $scores{$name}->{VOTES}->{$part}->{SCORE} = -2;
          }
          else {
            $scores{$name}->{VOTES}->{$part}->{SCORE} = -1;
          }
        }
        elsif ($votes{VOTES}->{$part}->{REC} eq 'against') {
          if ($scores{$name}->{VOTES}->{$part}->{VOTE} eq 'for') {
            $scores{$name}->{VOTES}->{$part}->{SCORE} = -2;
          }
          elsif ($scores{$name}->{VOTES}->{$part}->{VOTE} eq 'against') {
            $scores{$name}->{VOTES}->{$part}->{SCORE} = 2;
          }
          else {
            $scores{$name}->{VOTES}->{$part}->{SCORE} = 1;
          }
        }
      }
      else {
        # Speeches are special
        $scores{$name}->{VOTES}->{$part}->{SCORE} = ($scores{$name}->{VOTES}->{$part}->{VOTE} / 5) - 2;
      }
      # Weight
      $scores{$name}->{VOTES}->{$part}->{SCORE} *= $votes{VOTES}->{$part}->{WEIGHT};
    }
  }
};

if (my $e = catch std::exception) {
  warn "getDocument failed with exception\n";
  warn $e->what() . "\n";
  exit( -1 );
}
elsif ($@) {
  warn "getDocument failed\n";
  warn $@;
  exit( -1 );
}

# Sump up weights for all votes
$votes{SUM_WEIGHTS} = 0;
foreach my $val (values %{$votes{VOTES}}) {
  $votes{SUM_WEIGHTS} += $val->{WEIGHT} if $val->{WEIGHT};
}

my $an_score = 0;

foreach my $mp (keys %scores) {
  $scores{$mp}->{SUM_SCORES} = 0;
  # Count no vote as abstention
  # unless there is no recommendation
  foreach my $part (keys %{$votes{VOTES}}) {
    unless ($scores{$mp}->{VOTES}->{$part}) {
      if ($part ne 'Interventions en séance') {
        next unless $votes{VOTES}->{$part}->{REC};
        if ($votes{VOTES}->{$part}->{REC} eq 'for') {
          $scores{$mp}->{VOTES}->{$part}->{SCORE} = -1 * $votes{VOTES}->{$part}->{WEIGHT};
        }
        elsif ($votes{VOTES}->{$part}->{REC} eq 'against') {
          $scores{$mp}->{VOTES}->{$part}->{SCORE} = $votes{VOTES}->{$part}->{WEIGHT};
        }
      }
      else {
          $scores{$mp}->{VOTES}->{$part}->{SCORE} = 0;
      }
    }
  }

  # Sump up scores for all votes
  foreach my $val (values %{$scores{$mp}->{VOTES}}) {
    $scores{$mp}->{SUM_SCORES} += $val->{SCORE} if $val and $val->{SCORE};
  }

  # Compute final score from 0 to 20
  $scores{$mp}->{FINAL_SCORE} = sprintf("%.2f", 5 * ($scores{$mp}->{SUM_SCORES} / $votes{SUM_WEIGHTS}) + 10);
  $an_score += 5 * ($scores{$mp}->{SUM_SCORES} / $votes{SUM_WEIGHTS}) + 10;
}
$an_score = sprintf("%.2f", $an_score / scalar(keys %scores));

#use Data::Dumper;
#print "VOTES=!".Dumper(\%votes)."!\nSCORES=!".Dumper(\%scores)."!\n";

# Build main mediawiki page
my $plural = ($votes{TEXT} =~ / \/ /o) ? 's' : '';
my $mediawiki_page = << "MEDIAWIKI_PAGE";
__NOEDITSECTION__

==[[Memoire_politique|Mémoire politique]] : $vote_label==

Vote sur le$plural texte$plural '''$votes{TEXT}''', le $votes{DATE}, résultats des interventions en séances et des scrutins publics sur les parties détaillées ci-dessous.

Les scrutins publics publiés par l'Assemblée nationale ne détaillant pas le nom des votants lorsque ceux-ci votent conformément à la position majoritaire de leur groupe parlementaire, il est impossible de connaître exactement quel député a voté quoi sur ces amendements, articles ou motions. De plus, tous les amendements ne font pas l'objet de scrutins publics. Aussi, il n'a été tenu compte que des scrutins solennels et des interventions en séance.

Chaque intervention en séance a été notée entre 0 et 20. Cette note subjective tient compte de divers critères : mention des principaux avantages ou problèmes du projet de loi, degré d'approbation ou de contestation du projet de loi, argumentation basée sur des faits objectifs primordiaux ou au contraire sur des contre-vérités factuelles ou juridiques, attachement à la défense des citoyens ou d'intérêts particuliers. En outre, les interventions à partir du lundi 30 mars 2009 ayant été évaluées après que le Conseil constitutionnel a jugé dans sa décision du 10 juin 2009 que certaines dispositions du texte voté étaient frappées d'inconstitutionnalité, il a été tenu compte dans cette notation de la dénonciation ou au contraire du soutien à de telles mesures inconstitutionnelles. Par ailleurs, il a été tenu compte de l'implication dans le débats des députés : pour des interventions comparables, un député n'étant intervenu qu'à de rares occasions sera moins bien noté qu'un parlementaire présent à la quasi totalité des séances. Enfin, après le rejet en première lecture le 9 avril 2009 du texte issu de la Commission mixte paritaire (CMP), les notes sanctionnent également les interventions répondant en détail sur le fond et celles tentant d'expédier cette nouvelle lecture, notamment par le rapporteur ou le président de la commission des lois.

Les notes sur chaque intervention sont détaillées sur [[${vote_wiki}_interventions_en_seance]] et toutes les interventions peuvent être consultées sur la page associée à chaque député.

===Résultats===

'''Note moyenne de l'Assemblée nationale : $an_score/20'''

* [[${vote_wiki}_par_note|Résultats classés par note finale]]
* [[${vote_wiki}_par_nom|Résultats classés par ordre alphabétique]]
* [[${vote_wiki}_par_departement|Résultats classés par département]]
* [[${vote_wiki}_par_groupe|Résultats classés par groupe politique]]

===Scrutins publics et interventions en séance===

MEDIAWIKI_PAGE

my $details = '';
foreach my $part (@parts) {
  if ($part ne 'Interventions en séance') {
    my $rec =  $votes{VOTES}->{$part}->{REC} eq 'for' ? 'vote pour' :  $votes{VOTES}->{$part}->{REC} eq 'against' ? 'vote contre' : 'aucune';
    $details .= "* $part : $votes{VOTES}->{$part}->{DESCR} - recommandation : $rec".($votes{VOTES}->{$part}->{WEIGHT} ? ", coefficient : $votes{VOTES}->{$part}->{WEIGHT}" : '')."\n";
  }
  else {
    $details .= "* $part, note moyenne : $votes{VOTES}->{$part}->{DESCR} - coefficient : $votes{VOTES}->{$part}->{WEIGHT}\n";
  }
}
$mediawiki_page .= $details;

$mediawiki_page .= << "MEDIAWIKI_PAGE";

----

Retour à la page d'accueil de [[Memoire_politique|Mémoire politique]]
$categories{$vote_wiki}
MEDIAWIKI_PAGE

# Output main mediawiki page
my $mediawiki_location = "$output_dir/$vote_wiki.mediawiki";
open MEDIAWIKI, ">$mediawiki_location" or die "Cannot write in file $mediawiki_location: $!\n";
print MEDIAWIKI $mediawiki_page;
close MEDIAWIKI or die "Error when closing $mediawiki_location: $!\n";
warn "$mediawiki_location done\n";

# Build mediawiki page by score
$mediawiki_page = << "MEDIAWIKI_PAGE";
__NOEDITSECTION__

==[[Memoire_politique|Mémoire politique]] : $vote_label, résultats$detail_text par note finale==

MEDIAWIKI_PAGE

$mediawiki_page .= "{| border='1' cellpadding='5' cellspacing='0' class='wikitable sortable'\n";
$mediawiki_page .= "! Député(e) !! Note finale !! ".(join(' !! ', @parts))."\n";
$mediawiki_page .= "|-\n";
foreach my $mp (sort sort_by_score keys %scores) {
  my @coloured_scores = get_coloured_score($mp);
  my $wiki_mp = $scores{$mp}->{DBXMLID} ? "[[$scores{$mp}->{DBXMLID}|$mp]]" : $mp;
  my $id_mp = $scores{$mp}->{DBXMLID} ? "id='$scores{$mp}->{DBXMLID}'" : "id='$mp'";
  $id_mp =~ s/ /_/go;
  $mediawiki_page .= "|-$id_mp style='display:none'\n";
  $mediawiki_page .= "| $wiki_mp || style=\"background:".$colours[sprintf("%.0f", $scores{$mp}->{FINAL_SCORE})]."\" | $scores{$mp}->{FINAL_SCORE} || ".(join(' || ', @coloured_scores))."\n";
}
$mediawiki_page .= "|}\n";
if ($detail) {
  $mediawiki_page .= "\n[http://www.laquadrature.net/wiki/${vote_wiki}${detail_wiki}_par_note Voir les détails pour tous les députés]\n";
  $mediawiki_page .= "\n----\n\n$details\n";
  $mediawiki_page .= "\n-----\n\n[[${vote_wiki}_par_note|Revenir aux résultats par note finale]]\n";
}
else {
  $mediawiki_page .= "\n-----\n\n[[$vote_wiki|Autres classements sur ce vote]]\n";
}
$mediawiki_page .= "$categories{$vote_wiki}\n";

# Output mediawiki page by score
$mediawiki_location = "$output_dir/${vote_wiki}${detail_wiki}_par_note.mediawiki";
open MEDIAWIKI, ">$mediawiki_location" or die "Cannot write in file $mediawiki_location: $!\n";
print MEDIAWIKI $mediawiki_page;
close MEDIAWIKI or die "Error when closing $mediawiki_location: $!\n";
warn "$mediawiki_location done\n";

# Build global mediawiki page by score
if ($detail) {
  $mediawiki_page = << "MEDIAWIKI_PAGE";
__NOEDITSECTION__

==[[Memoire_politique|Mémoire politique]] : $vote_label, résultats par note finale==

* [[${vote_wiki}${detail_wiki}_par_note|Détail des notes]]

MEDIAWIKI_PAGE

  $mediawiki_page .= "{| border='1' cellpadding='5' cellspacing='0' class='wikitable sortable'\n";
  $mediawiki_page .= "! Rang !! Député(e) !! Département !! Groupe !! Note finale !!\n";
  $mediawiki_page .= "|-\n";
  my $rank = 1;
  foreach my $mp (sort sort_by_score keys %scores) {
    my @coloured_scores = get_coloured_score($mp);
    my $wiki_mp = $scores{$mp}->{DBXMLID} ? "[[$scores{$mp}->{DBXMLID}|$mp]]" : $mp;
    my $show = $scores{$mp}->{DBXMLID} ? "?showmep=$scores{$mp}->{DBXMLID}" : "?showmep=$mp";
    $show =~ s/ /_/go;
    my $department = "[[${vote_wiki}_$scores{$mp}->{DEPARTMENT}->{NUMBER}|$scores{$mp}->{DEPARTMENT}->{NAME}]]";
    my $grp_wiki = $scores{$mp}->{DBXMLID} ? $scores{$mp}->{GRP}->{CODE} : '?';
    $grp_wiki =~ s/[\/].+$//o;
    my $grp = "[[${vote_wiki}_$grp_wiki|$scores{$mp}->{GRP}->{CODE}]]";
    $mediawiki_page .= "| $rank || $wiki_mp || $department || $grp || style=\"background:".$colours[sprintf("%.0f", $scores{$mp}->{FINAL_SCORE})]."\" | $scores{$mp}->{FINAL_SCORE} || [http://www.laquadrature.net/wiki/${vote_wiki}${detail_wiki}_par_note$show Voir les details]\n";
    $mediawiki_page .= "|-\n";
    $rank++;
  }
  $mediawiki_page =~ s/\|-\n$/|}\n/so;
  $mediawiki_page .= "\n-----\n\n[[$vote_wiki|Autres classements sur ce vote]]\n";
  $mediawiki_page .= "$categories{$vote_wiki}\n";

  # Output global mediawiki page by score
  $mediawiki_location = "$output_dir/${vote_wiki}_par_note.mediawiki";
  open MEDIAWIKI, ">$mediawiki_location" or die "Cannot write in file $mediawiki_location: $!\n";
  print MEDIAWIKI $mediawiki_page;
  close MEDIAWIKI or die "Error when closing $mediawiki_location: $!\n";
  warn "$mediawiki_location done\n";
}

# Build mediawiki page by name
$mediawiki_page = << "MEDIAWIKI_PAGE";
__NOEDITSECTION__

==[[Memoire_politique|Mémoire politique]] : $vote_label, résultats$detail_text par ordre alphabétique==

MEDIAWIKI_PAGE

$mediawiki_page .= "{| border='1' cellpadding='5' cellspacing='0' class='wikitable sortable'\n";
$mediawiki_page .= "! Député(e) !! Note finale !! ".(join(' !! ', @parts))."\n";
foreach my $mp (sort sort_by_name keys %scores) {
  my @coloured_scores = get_coloured_score($mp);
  my $id_mp = $scores{$mp}->{DBXMLID} ? "id='$scores{$mp}->{DBXMLID}'" : "id='$mp'";
  $id_mp =~ s/ /_/go;
  $mediawiki_page .= "|-$id_mp style='display:none'\n";
  my $wiki_mp = $scores{$mp}->{DBXMLID} ? "[[$scores{$mp}->{DBXMLID}|$mp]]" : $mp;
  $mediawiki_page .= "| $wiki_mp || style=\"background:".$colours[sprintf("%.0f", $scores{$mp}->{FINAL_SCORE})]."\" | $scores{$mp}->{FINAL_SCORE} || ".(join(' || ', @coloured_scores))."\n";
}
$mediawiki_page .= "|}\n";
if ($detail) {
  $mediawiki_page .= "\n[http://www.laquadrature.net/wiki/${vote_wiki}${detail_wiki}_par_nom Voir les détails pour tous les députés]\n";
  $mediawiki_page .= "\n----\n\n$details\n";
  $mediawiki_page .= "\n-----\n\n[[${vote_wiki}_par_nom|Revenir aux résultats par ordre alphabétique]]\n";
}
else {
  $mediawiki_page .= "\n-----\n\n[[$vote_wiki|Autres classements sur ce vote]]\n";
}
$mediawiki_page .= "$categories{$vote_wiki}\n";

# Output mediawiki page by score
$mediawiki_location = "$output_dir/${vote_wiki}${detail_wiki}_par_nom.mediawiki";
open MEDIAWIKI, ">$mediawiki_location" or die "Cannot write in file $mediawiki_location: $!\n";
print MEDIAWIKI $mediawiki_page;
close MEDIAWIKI or die "Error when closing $mediawiki_location: $!\n";
warn "$mediawiki_location done\n";

# Build global mediawiki page by name
if ($detail) {
  $mediawiki_page = << "MEDIAWIKI_PAGE";
__NOEDITSECTION__

==[[Memoire_politique|Mémoire politique]] : $vote_label, résultats par ordre alphabétique==

* [[${vote_wiki}${detail_wiki}_par_nom|Détail des notes]]

MEDIAWIKI_PAGE

  $mediawiki_page .= "{| border='1' cellpadding='5' cellspacing='0' class='wikitable sortable'\n";
  $mediawiki_page .= "! Rang !! Député(e) !! Département !! Groupe !! Note finale !!\n";
  $mediawiki_page .= "|-\n";
  my $rank = 1;
  foreach my $mp (sort sort_by_name keys %scores) {
    my @coloured_scores = get_coloured_score($mp);
    my $wiki_mp = $scores{$mp}->{DBXMLID} ? "[[$scores{$mp}->{DBXMLID}|$mp]]" : $mp;
    my $show = $scores{$mp}->{DBXMLID} ? "?showmep=$scores{$mp}->{DBXMLID}" : "?showmep=$mp";
    $show =~ s/ /_/go;
    my $department = "[[${vote_wiki}_$scores{$mp}->{DEPARTMENT}->{NUMBER}|$scores{$mp}->{DEPARTMENT}->{NAME}]]";
    my $grp_wiki = $scores{$mp}->{DBXMLID} ? $scores{$mp}->{GRP}->{CODE} : '?';
    $grp_wiki =~ s/[\/].+$//o;
    my $grp = "[[${vote_wiki}_$grp_wiki|$scores{$mp}->{GRP}->{CODE}]]";
    $mediawiki_page .= "| $rank || $wiki_mp || $department || $grp || style=\"background:".$colours[sprintf("%.0f", $scores{$mp}->{FINAL_SCORE})]."\" | $scores{$mp}->{FINAL_SCORE} || [http://www.laquadrature.net/wiki/${vote_wiki}${detail_wiki}_par_nom$show Voir les détails]\n";
    $mediawiki_page .= "|-\n";
    $rank++;
  }
  $mediawiki_page =~ s/\|-\n$/|}\n/so;
  $mediawiki_page .= "\n-----\n\n[[$vote_wiki|Autres classements sur ce vote]]\n";
  $mediawiki_page .= "$categories{$vote_wiki}\n";

  # Output mediawiki page by score
  $mediawiki_location = "$output_dir/${vote_wiki}_par_nom.mediawiki";
  open MEDIAWIKI, ">$mediawiki_location" or die "Cannot write in file $mediawiki_location: $!\n";
  print MEDIAWIKI $mediawiki_page;
  close MEDIAWIKI or die "Error when closing $mediawiki_location: $!\n";
  warn "$mediawiki_location done\n";
}

# Build mediawiki pages for each department
my %department_mps = map {$_->{DEPARTMENT}->{NUMBER} => $_->{DEPARTMENT}->{NAME}} grep {$_->{DEPARTMENT}} values %scores;
my %department_mean;
foreach my $number (sort keys %department_mps) {
  my $preposition = $department_preposition{$number};
  my $mediawiki_page = << "MEDIAWIKI_PAGE";
__NOEDITSECTION__

==[[Memoire_politique|Mémoire politique]] : $vote_label, résultats$detail_text pour $preposition$department_mps{$number}==

MEDIAWIKI_PAGE

  $mediawiki_page .= "{| border='1' cellpadding='5' cellspacing='0' class='wikitable sortable'\n";
  $mediawiki_page .= "! Député(e) !! Note finale !! ".(join(' !! ', @parts))."\n";
  my $department_nb = 0;
  my $department_total = 0;
  foreach my $mp (sort sort_by_score grep {$scores{$_}->{DEPARTMENT} and $scores{$_}->{DEPARTMENT}->{NUMBER} eq $number} keys %scores) {
    my @coloured_scores = get_coloured_score($mp);
    my $id_mp = $scores{$mp}->{DBXMLID} ? "id='$scores{$mp}->{DBXMLID}'" : "id='$mp'";
    $id_mp =~ s/ /_/go;
    $mediawiki_page .= "|-$id_mp style='display:none'\n";
    my $wiki_mp = $scores{$mp}->{DBXMLID} ? "[[$scores{$mp}->{DBXMLID}|$mp]]" : $mp;
    $mediawiki_page .= "| $wiki_mp || style=\"background:".$colours[sprintf("%.0f", $scores{$mp}->{FINAL_SCORE})]."\" | $scores{$mp}->{FINAL_SCORE} || ".(join(' || ', @coloured_scores))."\n";
    $department_nb++;
    $department_total += $scores{$mp}->{FINAL_SCORE};
  }
  $mediawiki_page .= "|}\n";
  if ($detail) {
    $mediawiki_page .= "\n[http://www.laquadrature.net/wiki/${vote_wiki}${detail_wiki}_$number Voir les détails pour tous les députés]\n";
    $mediawiki_page .= "\n----\n\n$details\n";
    $mediawiki_page .= "\n-----\n\n[[${vote_wiki}_$number|Revenir aux résultats $preposition $department_mps{$number}]]\n";
  }
  else {
    $mediawiki_page .= "\n-----\n\n[[${vote_wiki}_par_departement|Autres départements]]\n";
  }
  $mediawiki_page .= "$categories{$vote_wiki}\n";
  $department_mean{$number} = sprintf("%.2f", $department_total / $department_nb);

  # Output mediawiki page for this department
  my $mediawiki_location = "$output_dir/${vote_wiki}${detail_wiki}_$number.mediawiki";
  open MEDIAWIKI, ">$mediawiki_location" or die "Cannot write in file $mediawiki_location: $!\n";
  print MEDIAWIKI $mediawiki_page;
  close MEDIAWIKI or die "Error when closing $mediawiki_location: $!\n";
  warn "$mediawiki_location done\n";

  # Build global mediawiki page for this department
  if ($detail) {
    my $mediawiki_page = << "MEDIAWIKI_PAGE";
__NOEDITSECTION__

==[[Memoire_politique|Mémoire politique]] : $vote_label, résultats $preposition $department_mps{$number}==

* [[${vote_wiki}${detail_wiki}_$number|Détail des notes]]

MEDIAWIKI_PAGE

    $mediawiki_page .= "{| border='1' cellpadding='5' cellspacing='0' class='wikitable sortable'\n";
    $mediawiki_page .= "! Rang !! Député(e) !! Groupe !! Note finale !!\n";
    $mediawiki_page .= "|-\n";
    my $rank = 1;
    foreach my $mp (sort sort_by_score grep {$scores{$_}->{DEPARTMENT} and $scores{$_}->{DEPARTMENT}->{NUMBER} eq $number} keys %scores) {
      my @coloured_scores = get_coloured_score($mp);
      my $wiki_mp = $scores{$mp}->{DBXMLID} ? "[[$scores{$mp}->{DBXMLID}|$mp]]" : $mp;
      my $show = $scores{$mp}->{DBXMLID} ? "?showmep=$scores{$mp}->{DBXMLID}" : "?showmep=$mp";
      $show =~ s/ /_/go;
      my $grp_wiki = $scores{$mp}->{DBXMLID} ? $scores{$mp}->{GRP}->{CODE} : '?';
    $grp_wiki =~ s/[\/].+$//o;
      my $grp = "[[${vote_wiki}_$grp_wiki|$scores{$mp}->{GRP}->{CODE}]]";
      $mediawiki_page .= "| $rank || $wiki_mp || $grp || style=\"background:".$colours[sprintf("%.0f", $scores{$mp}->{FINAL_SCORE})]."\" | $scores{$mp}->{FINAL_SCORE} || [http://www.laquadrature.net/wiki/${vote_wiki}${detail_wiki}_$number$show Voir les détails]\n";
      $mediawiki_page .= "|-\n";
      $rank++;
    }
    $mediawiki_page =~ s/\|-\n$/|}\n/so;
    $mediawiki_page .= "\n-----\n\n[[${vote_wiki}_par_departement|Autres départements]]\n";
    $mediawiki_page .= "$categories{$vote_wiki}\n";

    # Output mediawiki page for this department
    my $mediawiki_location = "$output_dir/${vote_wiki}_$number.mediawiki";
    open MEDIAWIKI, ">$mediawiki_location" or die "Cannot write in file $mediawiki_location: $!\n";
    print MEDIAWIKI $mediawiki_page;
    close MEDIAWIKI or die "Error when closing $mediawiki_location: $!\n";
    warn "$mediawiki_location done\n";
  }
}

# Build mediawiki page by department
$mediawiki_page = << "MEDIAWIKI_PAGE";
__NOEDITSECTION__

==[[Memoire_politique|Mémoire politique]] : $vote_label, résultats par département==

{|- style="background:none;"
MEDIAWIKI_PAGE

my $nb_col = 1;
my $col = 0;
foreach my $department_number (sort  {$department_mean{$b} <=> $department_mean{$a}} keys %department_mps) {
  $mediawiki_page .= "| [[${vote_wiki}_$department_number|$department_mps{$department_number}]]: note moyennne&nbsp;: $department_mean{$department_number}/20\n";
  $col++;
  if ($col == $nb_col) {
    $mediawiki_page .= "|-\n";
    $col = 0;
  }
}
$mediawiki_page =~ s/|-\n$//o;
$mediawiki_page .= "|}\n";
$mediawiki_page .= "\n-----\n\n[[$vote_wiki|Autres classements sur ce vote]]\n";
$mediawiki_page .= "$categories{$vote_wiki}\n";

# Output mediawiki page by department
$mediawiki_location = "$output_dir/${vote_wiki}_par_departement.mediawiki";
open MEDIAWIKI, ">$mediawiki_location" or die "Cannot write in file $mediawiki_location: $!\n";
print MEDIAWIKI $mediawiki_page;
close MEDIAWIKI or die "Error when closing $mediawiki_location: $!\n";
warn "$mediawiki_location done\n";

# Build mediawiki pages for each group
my %group_mps = map {$_->{GRP}->{CODE} => $_->{GRP}->{NAME}} grep {$_->{GRP}} values %scores;
my %group_mean;
foreach my $code (sort keys %group_mps) {
  my $mediawiki_page = << "MEDIAWIKI_PAGE";
__NOEDITSECTION__

==[[Memoire_politique|Mémoire politique]] : $vote_label, résultats$detail_text pour le groupe $group_mps{$code}==

MEDIAWIKI_PAGE

  $mediawiki_page .= "{| border='1' cellpadding='5' cellspacing='0' class='wikitable sortable'\n";
  $mediawiki_page .= "! Député(e) !! Note finale !! ".(join(' !! ', @parts))."\n";
  my $group_nb = 0;
  my $group_total = 0;
  foreach my $mp (sort sort_by_score grep {$scores{$_}->{GRP} and $scores{$_}->{GRP}->{CODE} =~ /^$code/} keys %scores) {
    my @coloured_scores = get_coloured_score($mp);
    my $wiki_mp = $scores{$mp}->{DBXMLID} ? "[[$scores{$mp}->{DBXMLID}|$mp]]" : $mp;
    my $id_mp = $scores{$mp}->{DBXMLID} ? "id='$scores{$mp}->{DBXMLID}'" : "id='$mp'";
    $id_mp =~ s/ /_/go;
    $mediawiki_page .= "|-$id_mp style='display:none'\n";
    $mediawiki_page .= "| $wiki_mp || style=\"background:".$colours[sprintf("%.0f", $scores{$mp}->{FINAL_SCORE})]."\" | $scores{$mp}->{FINAL_SCORE} || ".(join(' || ', @coloured_scores))."\n";
    $group_nb++;
    $group_total += $scores{$mp}->{FINAL_SCORE};
  }
  $mediawiki_page .= "|}\n";
  if ($detail) {
    $mediawiki_page .= "\n[http://www.laquadrature.net/wiki/${vote_wiki}${detail_wiki}_$code Voir les détails pour tous les députés]\n";
    $mediawiki_page .= "\n----\n\n$details\n";
    $mediawiki_page .= "\n-----\n\n[[${vote_wiki}_$code|Revenir aux résultats pour le groupe $group_mps{$code}]]\n";
  }
  else {
    $mediawiki_page .= "\n-----\n\n[[${vote_wiki}_par_groupe|Autres groupes politiques]]\n";
  }
  $mediawiki_page .= "$categories{$vote_wiki}\n";
  $group_mean{$code} = sprintf("%.2f", $group_total / $group_nb);

  # Output mediawiki page for this group
  my $mediawiki_location = "$output_dir/${vote_wiki}${detail_wiki}_$code.mediawiki";
  open MEDIAWIKI, ">$mediawiki_location" or die "Cannot write in file $mediawiki_location: $!\n";
  print MEDIAWIKI $mediawiki_page;
  close MEDIAWIKI or die "Error when closing $mediawiki_location: $!\n";
  warn "$mediawiki_location done\n";

  # Build global mediawiki page for this group
  if ($detail) {
    my $mediawiki_page = << "MEDIAWIKI_PAGE";
__NOEDITSECTION__

==[[Memoire_politique|Mémoire politique]] : $vote_label, résultats pour le groupe $group_mps{$code}==

* [[${vote_wiki}${detail_wiki}_$code|Détail des notes]]

MEDIAWIKI_PAGE

    $mediawiki_page .= "{| border='1' cellpadding='5' cellspacing='0' class='wikitable sortable'\n";
    $mediawiki_page .= "! Rang !! Député(e) !! Département !! Note finale !!\n";
    $mediawiki_page .= "|-\n";
    my $rank = 1;
    foreach my $mp (sort sort_by_score grep {$scores{$_}->{GRP} and $scores{$_}->{GRP}->{CODE} =~ /^$code/} keys %scores) {
      my @coloured_scores = get_coloured_score($mp);
      my $wiki_mp = $scores{$mp}->{DBXMLID} ? "[[$scores{$mp}->{DBXMLID}|$mp]]" : $mp;
      my $show = $scores{$mp}->{DBXMLID} ? "?showmep=$scores{$mp}->{DBXMLID}" : "?showmep=$mp";
      $show =~ s/ /_/go;
      my $department = "[[${vote_wiki}_$scores{$mp}->{DEPARTMENT}->{NUMBER}|$scores{$mp}->{DEPARTMENT}->{NAME}]]";
      $mediawiki_page .= "| $rank || $wiki_mp || $department || style=\"background:".$colours[sprintf("%.0f", $scores{$mp}->{FINAL_SCORE})]."\" | $scores{$mp}->{FINAL_SCORE} || [http://www.laquadrature.net/wiki/${vote_wiki}${detail_wiki}_$code$show Voir les détails]\n";
      $mediawiki_page .= "|-\n";
      $rank++;
    }
    $mediawiki_page =~ s/\|-\n$/|}\n/so;
    $mediawiki_page .= "\n-----\n\n[[${vote_wiki}_par_groupe|Autres groupes politiques]]\n";
    $mediawiki_page .= "$categories{$vote_wiki}\n";

    # Output mediawiki page for this group
    my $mediawiki_location = "$output_dir/${vote_wiki}_$code.mediawiki";
    open MEDIAWIKI, ">$mediawiki_location" or die "Cannot write in file $mediawiki_location: $!\n";
    print MEDIAWIKI $mediawiki_page;
    close MEDIAWIKI or die "Error when closing $mediawiki_location: $!\n";
    warn "$mediawiki_location done\n";
  }
}

# Build mediawiki page by group
$mediawiki_page = << "MEDIAWIKI_PAGE";
__NOEDITSECTION__

==[[Memoire_politique|Mémoire politique]] : $vote_label, résultats par groupe politique==

{|- style="background:none;"
MEDIAWIKI_PAGE

$nb_col = 1;
$col = 0;
foreach my $group_code (sort { $group_mean{$b} <=> $group_mean{$a} } keys %group_mps) {
  $mediawiki_page .= "| [[${vote_wiki}_$group_code|$group_mps{$group_code}]]: note moyennne&nbsp;: $group_mean{$group_code}/20\n";
  $col++;
  if ($col == $nb_col) {
    $mediawiki_page .= "|-\n";
    $col = 0;
  }
}
$mediawiki_page =~ s/|-\n$//o;
$mediawiki_page .= "|}\n";
$mediawiki_page .= "\n-----\n\n[[$vote_wiki|Autres classements sur ce vote]]\n";
$mediawiki_page .= "$categories{$vote_wiki}\n";

# Output mediawiki page by group
$mediawiki_location = "$output_dir/${vote_wiki}_par_groupe.mediawiki";
open MEDIAWIKI, ">$mediawiki_location" or die "Cannot write in file $mediawiki_location: $!\n";
print MEDIAWIKI $mediawiki_page;
close MEDIAWIKI or die "Error when closing $mediawiki_location: $!\n";
warn "$mediawiki_location done\n";

sub get_coloured_score {
  my $mp = shift;
  my @coloured_scores;

  foreach my $part (@parts) {
    my $colour = '#ffffff';
    my $choice = '-';
    if ($part ne 'Interventions en séance') {
      if ($votes{VOTES}->{$part}->{REC}) {
        if ($scores{$mp}->{VOTES}->{$part}->{SCORE} / $votes{VOTES}->{$part}->{WEIGHT} == 2) {
          $colour = '#00ff00'
        }
        elsif ($scores{$mp}->{VOTES}->{$part}->{SCORE} / $votes{VOTES}->{$part}->{WEIGHT} == 1) {
          $colour = '#008800'
        }
        elsif ($scores{$mp}->{VOTES}->{$part}->{SCORE} / $votes{VOTES}->{$part}->{WEIGHT} == -1) {
          $colour = '#dc143c'
        }
        elsif ($scores{$mp}->{VOTES}->{$part}->{SCORE} / $votes{VOTES}->{$part}->{WEIGHT} == -2) {
          $colour = '#ff0000'
        }
      }

      if ($scores{$mp}->{VOTES}->{$part}->{VOTE}) {
        if ($scores{$mp}->{VOTES}->{$part}->{VOTE} eq 'for') {
          $choice = 'pour';
        }
        elsif ($scores{$mp}->{VOTES}->{$part}->{VOTE} eq 'against') {
          $choice = 'contre';
        }
        elsif ($scores{$mp}->{VOTES}->{$part}->{VOTE} eq 'abstention') {
          $choice = 'abst.';
        }
      }
    }
    else {
      $choice = (5 * $scores{$mp}->{VOTES}->{$part}->{SCORE} / $votes{VOTES}->{$part}->{WEIGHT}) + 10; 
      $colour = $colours[sprintf("%.0f", $choice)];
    }

    push @coloured_scores, "style=\"background:$colour\" | $choice";
  }
  return @coloured_scores;
}

sub sort_by_score {
  $scores{$b}->{FINAL_SCORE} <=> $scores{$a}->{FINAL_SCORE} ||
  (
    $scores{$a}->{NAME} ?
      (
        $scores{$b}->{NAME} ?
          (
            $scores{$a}->{NAME}->{LAST} cmp $scores{$b}->{NAME}->{LAST} ||
            $scores{$a}->{NAME}->{FIRST} cmp $scores{$b}->{NAME}->{FIRST}
          ) :
        $scores{$a}->{NAME}->{LAST} cmp $b
      ) :
      (
        $scores{$b}->{NAME} ?
          $a cmp $scores{$b}->{NAME}->{LAST} :
          $a cmp $b
      )
  )
}

sub sort_by_name {
  $scores{$a}->{NAME} ?
    (
      $scores{$b}->{NAME} ?
        (
          $scores{$a}->{NAME}->{LAST} cmp $scores{$b}->{NAME}->{LAST} ||
          $scores{$a}->{NAME}->{FIRST} cmp $scores{$b}->{NAME}->{FIRST}
        ) :
      $scores{$a}->{NAME}->{LAST} cmp $b
    ) :
    (
      $scores{$b}->{NAME} ?
        $a cmp $scores{$b}->{NAME}->{LAST} :
        $a cmp $b
    )
}

sub contains {
  my $array = shift;
  my $elt = shift;

  foreach my $this_elt (@$array) {
    return 1 if $this_elt eq $elt;
  }
  return 0;
}
