#!/usr/bin/perl

# Copyright 2007-2008 Gérald Sédrati-Dinet, gibus@april.org
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, see <http://www.gnu.org/licenses/>.

use strict;
use warnings;
use encoding 'utf8';

use Sleepycat::DbXml 'simple';
use XML::Twig;
use LWP::UserAgent;
use Getopt::Long;

# Some files will be save under tree hierarchy based on executable location
use FindBin qw($Bin);
use lib "$Bin";
(my $root_dir = $Bin) =~ s/\/[^\/]*$//o;

# Default values
my $path2DbEnv = "$root_dir/dbxml";

# Parse command line options
Getopt::Long::Configure("bundling");
GetOptions(
  'path2dbenv|p=s' => \$path2DbEnv,
  'help|h'         =>
    sub {
      print STDERR <<USAGE;
Usage: $0 [-h] [-p <path_to_db_env>] <container>
where <container> can be meps, en_meps or mps
(for convenience the final "s" can be omitted, also ".dbxml" can be appended)

Options can be:
  --path2dbenv, -p <path_to_db_env>: path to directory of DB XML database
                                     default: $root_dir/dbxml
  --help, -h:                        print this message
USAGE
      exit 0;
    }
);

my $theContainer = shift or die "You should provide the name of the container, see $0 -h\n";
$theContainer =~ s/s?(?:\.dbxml)?$/s/;

# Create a user agent object
my $ua = LWP::UserAgent->new;
$ua->agent("$0 (gibus perl script to fetch information from http://www.laquadrature.net/wiki/)");
my $headers = HTTP::Headers->new('Accept-Language' => 'en');

# Fetch the wiki page
my $wiki_url = "http://www.laquadrature.net/wiki/Written_Declaration_12/2010_signatories_list";
my $wiki_req = HTTP::Request->new(GET => $wiki_url, $headers);
my $wiki_res = $ua->request($wiki_req);
unless ($wiki_res->is_success) {
  die "Error fetching wiki page: $wiki_url: ", $wiki_res->status_line, "\n";
}
my $wiki_html = $wiki_res->content;
die "Unknown wiki page $wiki_url\n" if $wiki_html =~ /<div class="noarticletext">/o;

# Extract signatories
(my $signatories_str) = ($wiki_html =~ /<a name="List_of_the_signatories" id="List_of_the_signatories">(.+)<a name="List_of_the_non-signatories" id="List_of_the_non-signatories">/os);
my @signatories = ($signatories_str =~ /^<td> <a href="\/wiki\/(\w+)" title="\1">/gom);

my $nb_updated = 0;
foreach my $wiki_name (@signatories) {
  # Some Meps have change their names
  $wiki_name =~ s/IvayloKalfin/IvailoKalfin/o;
  $wiki_name =~ s/LenaBarbaraKolarskaBobinska/LenaKolarskaBobiska/o;
  $wiki_name =~ s/MariaPalomaMunizdeUrquiza/MariaMunizdeUrquiza/o;
  eval {
    # Open a container in the db environment
    my $env = new DbEnv(0);
    $env->set_cachesize(0, 64 * 1024, 1);
    $env->open($path2DbEnv, Db::DB_INIT_MPOOL|Db::DB_CREATE|Db::DB_INIT_LOCK|Db::DB_INIT_LOG);
    my $theMgr = new XmlManager($env);

    my $container = $theMgr->openContainer("$theContainer.dbxml");
    my $updateContext = $theMgr->createUpdateContext();

    # Get the document
    my $theDocument = $container->getDocument($wiki_name);
    my $docString = $theDocument->getContent();
    my $xml = XML::Twig->new(output_encoding=>'UTF-8',pretty_print => 'indented')->parse($docString);

    # Add signature as new opinions
    my $opinions_elt = $xml->root->first_child('opinions') ? $xml->root->first_child('opinions') : $xml->root->insert_new_elt('last_child','opinions');
    my $date_opinion = sprintf "%02d/%02d/%04d", sub {($_[3], $_[4]+1, $_[5]+1900)}->(localtime);
    my $title_opinion = 'Written Declaration 12/2010 (ACTA)';
    my $url_opinion = 'http://www.laquadrature.net/wiki/Written_Declaration_12/2010_signatories_list';
    my $opinion = 'Has signed wirtten declaration 12/2010 on the lack of a transparent process for the Anti-Counterfeiting Trade Agreement (ACTA) and potentially objectionable content.'; 

    # Skip if already added
    use Data::Dumper;
    return 0 if $opinions_elt->get_xpath('opinion/title[string()="Written Declaration 12/2010 (ACTA)"]');

    my $opinion_elt = $opinions_elt->insert_new_elt('first_child', 'opinion');
    $opinion_elt->insert_new_elt('last_child', 'date', $date_opinion);
    $opinion_elt->insert_new_elt('last_child', 'title', $title_opinion);
    $opinion_elt->insert_new_elt('last_child', 'url', $url_opinion);
    $opinion_elt->insert_new_elt('last_child', 'content', $opinion);

    # Update document into database
    $docString = $xml->sprint;
    $theDocument->setContent($docString);
    $container->updateDocument($theDocument, $updateContext);
    print "$wiki_name updated in $theContainer.dbxml.\n";
    $nb_updated++;

    return 0;
  } ;

  if (my $e = catch std::exception) {
    warn "Exception:\n";
    warn $e->what() . "\n";
    exit( -1 );
  }
  elsif ($@) {
    warn "Query failed\n";
    warn $@;
    exit( -1 );
  }
}

print "$nb_updated on " . scalar(@signatories) . " signatories updated\n";
