#!/usr/bin/perl

# Copyright 2007-2008 Gérald Sédrati-Dinet, gibus@april.org
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# any later version.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, see <http://www.gnu.org/licenses/>.

use strict;
use warnings;
use Sleepycat::DbXml 'simple';
use XML::Twig;
use Getopt::Long;
use utf8;
use open ':utf8';

# Some files will be save under tree hierarchy based on executable location
use FindBin qw($Bin);
use lib "$Bin";
(my $root_dir = $Bin) =~ s/\/[^\/]*$//o;

# Default values
my $path2DbEnv = "$root_dir/dbxml";
my $theContainer = 'mps_votes.dbxml';
my $theMpContainer = 'mps.dbxml';

# Parse command line options
Getopt::Long::Configure("bundling");
GetOptions(
  'path2dbenv|p=s'    => \$path2DbEnv,
  'container|c=s'     => \$theContainer,
  'mp_container|m=s'  => \$theMpContainer,
  'help|h'            =>
    sub {
      print STDERR <<USAGE;
Usage: $0 [-h] [-p <path_to_db_env>] [-c <container>]
Options can be:
  --path2dbenv, -p <path_to_db_env>: path to directory of DB XML database
                                     default: $root_dir/dbxml
  --container, -c <container>:       DB XML container for votes
                                     <container> is typically votes (for convenience the final "s" can be omitted, also ".dbxml" can be appended)
                                     default: mps_votes.dbxml
  --mp_container, -m <container>:    DB XML container for MPs
                                     <container> is typically mps (for convenience the final "s" can be omitted, also ".dbxml" can be appended)
                                     default: mps.dbxml
  --help, -h:                        print this message
USAGE
      exit 0;
    }
);
$theContainer =~ s/s?(?:\.dbxml)?$/s/;
$theMpContainer =~ s/s?(?:\.dbxml)?$/s/;

my %scores_mps;

# Open a container in the db environment
my $env = new DbEnv(0);
$env->set_cachesize(0, 64 * 1024, 1);
$env->open($path2DbEnv,
          Db::DB_INIT_MPOOL|Db::DB_CREATE|Db::DB_INIT_LOCK|Db::DB_INIT_LOG);
my $theMgr = new XmlManager($env);

my $container = $theMgr->openContainer("$theContainer.dbxml");
my $mp_container = $theMgr->openContainer("$theMpContainer.dbxml");

my $updateContext = $theMgr->createUpdateContext();

# Get all documents
eval {
  my $results = $container->getAllDocuments(0);
  my $value ;
  while( $results->next($value) ) {
    my %votes;
    my %scores;

    # Build xml tree
    my $xml = XML::Twig->new(output_encoding => 'UTF-8')->parse($value);
  
    # Extract infos
    my $vote_label = $xml->root->first_child_text('label');
    my $vote_wiki = $xml->root->first_child_text('wiki');
    my $date_elt = $xml->root->last_child('vote')->last_child('date');
    my $date = sprintf "%s/%s/%04d", $date_elt->first_child_text('day'), $date_elt->first_child_text('month'), $date_elt->first_child_text('year');
  
    foreach my $vote_elt ($xml->root->children('vote')) {
      my $vote_subject = $vote_elt->first_child('subject');
      my $text = $vote_subject->first_child_text('text');
      if (exists $votes{TEXT}) {
        my $txt_regexp = $text;
        $txt_regexp =~ s/([\(\)\[\]\+\*\.\?])/\\$1/go;
        unless ($votes{TEXT} =~ /$txt_regexp/) {
          $votes{TEXT} .= " / $text";
        }
      }
      else {
        $votes{TEXT} = $text;
      }

      my $part = $vote_subject->first_child_text('part');
      # Append text name in case of another text with the same part already exists
      $part .= "_$text" if $votes{VOTES}->{$part};
  
      $votes{VOTES}->{$part}->{REC} = $vote_subject->first_child_text('recommendation');
      $votes{VOTES}->{$part}->{WEIGHT} = $vote_subject->first_child_text('weight');
  
      my $result_elt = $vote_elt->first_child('result');
      foreach my $mp ($result_elt->children('mp')) {
        my $name = $mp->first_child_text('name');
        unless (exists $scores{$name}) {
          next unless $mp->first_child('dbxmlid');
          my $dbxmlid = $mp->first_child_text('dbxmlid');
          $scores{$name}->{DBXMLID} = $dbxmlid;
        }
        $scores{$name}->{VOTES}->{$part}->{VOTE} = $mp->first_child_text('choice');
       
        # Compute score for this vote
        if ($part ne 'Interventions en séance') {
        # unless there is no recommendation
          next unless $votes{VOTES}->{$part}->{REC};
          if ($votes{VOTES}->{$part}->{REC} eq 'for') {
            if ($scores{$name}->{VOTES}->{$part}->{VOTE} eq 'for') {
              $scores{$name}->{VOTES}->{$part}->{SCORE} = 2;
            }
            elsif ($scores{$name}->{VOTES}->{$part}->{VOTE} eq 'against') {
              $scores{$name}->{VOTES}->{$part}->{SCORE} = -2;
            }
            else {
              $scores{$name}->{VOTES}->{$part}->{SCORE} = -1;
            }
          }
          elsif ($votes{VOTES}->{$part}->{REC} eq 'against') {
            if ($scores{$name}->{VOTES}->{$part}->{VOTE} eq 'for') {
              $scores{$name}->{VOTES}->{$part}->{SCORE} = -2;
            }
            elsif ($scores{$name}->{VOTES}->{$part}->{VOTE} eq 'against') {
              $scores{$name}->{VOTES}->{$part}->{SCORE} = 2;
            }
            else {
              $scores{$name}->{VOTES}->{$part}->{SCORE} = 1;
            }
          }
        }
        else {
          # Speeches are special
          $scores{$name}->{VOTES}->{$part}->{SCORE} = ($scores{$name}->{VOTES}->{$part}->{VOTE} / 5) - 2;
        }
        # Weight
        $scores{$name}->{VOTES}->{$part}->{SCORE} *= $votes{VOTES}->{$part}->{WEIGHT};
      }
    }

    # Sump up weights for all votes
    $votes{SUM_WEIGHTS} = 0;
    foreach my $val (values %{$votes{VOTES}}) {
      $votes{SUM_WEIGHTS} += $val->{WEIGHT} if $val->{WEIGHT};
    }
    foreach my $mp (keys %scores) {
      $scores{$mp}->{SUM_SCORES} = 0;
      # Count no vote as abstention
      # unless there is no recommendation
      foreach my $part (keys %{$votes{VOTES}}) {
        unless ($scores{$mp}->{VOTES}->{$part}) {
          if ($part ne 'Interventions en séance') {
            next unless $votes{VOTES}->{$part}->{REC};
            if ($votes{VOTES}->{$part}->{REC} eq 'for') {
              $scores{$mp}->{VOTES}->{$part}->{SCORE} = -1 * $votes{VOTES}->{$part}->{WEIGHT};
            }
            elsif ($votes{VOTES}->{$part}->{REC} eq 'against') {
              $scores{$mp}->{VOTES}->{$part}->{SCORE} = $votes{VOTES}->{$part}->{WEIGHT};
            }
          }
          else {
            $scores{$mp}->{VOTES}->{$part}->{SCORE} = 0;
          }
        }
      }
    
      # Sump up scores for all votes
      foreach my $val (values %{$scores{$mp}->{VOTES}}) {
        $scores{$mp}->{SUM_SCORES} += $val->{SCORE} if $val and $val->{SCORE};
      }
    
      # Compute final score from 0 to 20
      $scores{$mp}->{FINAL_SCORE} = sprintf("%.2f", 5 * ($scores{$mp}->{SUM_SCORES} / $votes{SUM_WEIGHTS}) + 10);
      
      # Fill final structure
      $scores_mps{$mp}->{DBXMLID} = $scores{$mp}->{DBXMLID};
      $scores_mps{$mp}->{$vote_label}->{SCORE} = $scores{$mp}->{FINAL_SCORE};
      $scores_mps{$mp}->{$vote_label}->{WIKI} = $vote_wiki;
      $scores_mps{$mp}->{$vote_label}->{DATE} = $date;
    }
  }

  # Iterate through MP to update database
  foreach my $mp (keys %scores_mps) {
    # Get MP document
    my $mp_id = delete $scores_mps{$mp}->{DBXMLID};
    my $mp_document = $mp_container->getDocument($mp_id);
    my $mp_doc_string = $mp_document->getContent();
    my $mp_xml = XML::Twig->new(output_encoding=>'UTF-8',pretty_print => 'indented')->parse($mp_doc_string);

    # Delete old scores in the XML tree
    my $scores_elt = $mp_xml->root->first_child('scores') ? $mp_xml->root->first_child('scores') : $mp_xml->root->insert_new_elt('last_child','scores');
    $scores_elt->cut_children;

    # Add computed scores
    foreach my $score (sort {$scores_mps{$mp}->{$b}->{DATE} cmp $scores_mps{$mp}->{$a}->{DATE}} keys %{$scores_mps{$mp}}) {
      (my $printable_date = $scores_mps{$mp}->{$score}->{DATE}) =~ s/^(\d\d\d\d)(\/\d\d\/)(\d\d)/$3$2$1/o;
      my $score_elt = $scores_elt->insert_new_elt('last_child', 'score');
      $score_elt->insert_new_elt('last_child', 'label', $score);
      $score_elt->insert_new_elt('last_child', 'date', $printable_date);
      $score_elt->insert_new_elt('last_child', 'value', $scores_mps{$mp}->{$score}->{SCORE});
      $score_elt->insert_new_elt('last_child', 'wiki', $scores_mps{$mp}->{$score}->{WIKI});
    }

    # Update document into database
    $mp_doc_string = $mp_xml->sprint;
    $mp_document->setContent($mp_doc_string);
    $mp_container->updateDocument($mp_document, $updateContext);
    warn "$mp_id updated\n";
  }
};

if (my $e = catch std::exception) {
  warn "getDocument failed with exception\n";
  warn $e->what() . "\n";
  exit( -1 );
}
elsif ($@) {
  warn "getDocument failed\n";
  warn $@;
  exit( -1 );
}
